/*****************************************************************************/
/*                        demo plugin for reflections                        */
/*****************************************************************************/

/* title:    math tex demo plugin */
/* author:   Carsten Fuchs  */
/* version:  1.0, 16.9.97   */
/* updated:  10.3.98 by RG */
/*****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

/*****************************************************************************/
/* Includen aller notwendigen Dateien */
                                          
#include <bgw_all.h>

/*****************************************************************************/

#define vekcos(a,b) (a[0]*b[0]+a[1]*b[1]+a[2]*b[2])
#define vek_copy(a,b) {b[0]=a[0],b[1]=a[1],b[2]=a[2];}

void vekprod(float x[3],float y[3],float z[3])
/* Speichert in z das Vektorprodukt von x und y */
{
   z[0]=x[1]*y[2]-x[2]*y[1];
   z[1]=x[2]*y[0]-x[0]*y[2];
   z[2]=x[0]*y[1]-x[1]*y[0];
}

/*****************************************************************************/

   void 
proc_bump_n
   (float dm1,float dm2,float pu[3],float pv[3],float e[3],float n[3])

/* Dreht den Vektor n entsprechend den Winkeln dm1 und dm2. Die 
   Trefferkoordinaten holt er aus last_u und last_v, die bei der
   normalen Texturberechnung zwischengespeichert wurden.
   pu ist der u-, pv ist der v_vektor, beide liegen in der ebene e.
*/
{
   float ab,si,co,xneu,yneu,n3[3];
   float alpha_u,alpha_v;

   /* Zuerst n nach n1 projeziert */
   ab=1.0F; 
   n3[0]=0.0F;
   n3[1]=0.0F;
   n3[2]=ab;  /* Koordinaten von n bezglich pu,pv in der Ebene e */
  
   /* Jetzt die Steigung an der stelle u,v in
      pu- bzw. pv-Richtung errechnen */
   alpha_u=dm1;
   alpha_v=dm1;
   /* n wird um alpha_u um pu-Achse und alpha_v um pv-Achse gedreht */

   /* Zuerst u-achse */
   si=(float)sin(alpha_u); co=(float)cos(alpha_u);
   xneu=co*n3[0]-si*n3[2];
   yneu=si*n3[0]+co*n3[2];
   n3[0]=xneu;
   n3[2]=yneu;

   /* Jetzt v-achse */
   si=(float)sin(alpha_v); co=(float)cos(alpha_v);
   xneu=co*n3[1]-si*n3[2];
   yneu=si*n3[1]+co*n3[2];
   n3[1]=xneu;
   n3[2]=yneu;

   /* Jetzt wieder ins Weltkoordinatensystem zurckwandeln */
   n[0]=n3[0]*pu[0]+n3[1]*pv[0]+n3[2]*e[0];
   n[1]=n3[0]*pu[1]+n3[1]*pv[1]+n3[2]*e[1];
   n[2]=n3[0]*pu[2]+n3[1]*pv[2]+n3[2]*e[2];
}

/*****************************************************************************/

   static long 
lochbl
   (PLI_param *p)
{
   float *mat_param;
   proctex_rec *pt;
   float ab,dx,dy,u1,v1,u,v,du,dv,rad,du1,dv1;
   long i,j;

   pt = (proctex_rec *)p->v_param[1];
   mat_param = (float *)p->v_param[3];

   /* Wir schauen nur, ob innerhalb eines der Lcher.
      wenn ja, returnen wir 0: durchsichtig,
      sonst returnen wir 1: undurchsichtig
   */
   u=p->f_param[3];    /* textur-lokale u-Koordinate */
   v=p->f_param[4];    /* textur-lokale v-Koordinate */
   rad=pt->param[0];
   du=pt->param[1];
   dv=pt->param[2];
   if (du<=0.0001F)
     du=0.0001F;
   if (dv<=0.0001F)
     dv=0.0001F;
   du1=du*0.5F;
   if (u<0.0F)
     du1=-du1;
   dv1=dv*0.5F;
   if (v<0)
     dv1=-dv1;
   i=(long)((u+du1)/du);
   j=(long)((v+dv1)/dv);
   u1=du*(float)i;
   v1=dv*(float)j;
   
   dx=(u-u1);
   dy=(v-v1);
   ab=(float)sqrt((double)(dx*dx+dy*dy));
   if (ab<rad)
      return(0);
   return(1);
}

/*****************************************************************************/

static int kreisw_no_bump;  /* Flag, ob Bump bei kreiswelle */
static float kreisw_last_s_g[3];
static float kreisw_last_s_l[3];

   static long 
kreisw
   (PLI_param *p)
{
   PLI_param in,out;
   float *mat_param;
   proctex_rec *pt;
   
   pt = (proctex_rec *)p->v_param[1];
   mat_param = (float *)p->v_param[2];

   /* Wir schauen, ob es schon einen zuvor-Treffer gab */
   in.id            = BMS_GET_LAST_PTEX_HIT;
   in.l_param[0]    = p->l_param[0]; /* marke */
   in.v_param[0]    = p->v_param[0]; /* tex-handle */
    kreisw_no_bump=1;
   if (RPI_do(&in,&out))
   {
       if (out.l_param[0])
       {
          kreisw_last_s_g[0]=out.f_param[0];
          kreisw_last_s_g[1]=out.f_param[1];
          kreisw_last_s_g[2]=out.f_param[2];
          kreisw_last_s_l[0]=out.f_param[3];
          kreisw_last_s_l[1]=out.f_param[4];
          kreisw_last_s_l[2]=out.f_param[5];
          kreisw_no_bump=0;
       }
   }
   /* An Farbe und Oberflcheneigenschaften nichts verndern.
      Normalenvektor wird in kreisw_bump berechnet. */

   return(1);
}

/*****************************************************************************/

#define KREISW_HEIGHT(s,h,ampl,frequ,start)\
   {double abst,s1=s[0]-0.5,s2=s[1]-0.5;abst=sqrt(s1*s1+s2*s2);\
   h=(float)((double)ampl*sin((double)frequ*(abst+(double)start)));}

   static long 
kreisw_bump
   (PLI_param *p,float *n_neu)
{
    float e[4];
    float f,f1,hell,hell1,hell2;
    float n1[3],n2[3],s2[3],s2_tex[3];
    float n1_len,m1,m2,hilf;
    PLI_param in,out;
    float *s,*s_lokal,*n;
    proctex_rec *pt;
   
    if (kreisw_no_bump) /* Kein Bump ? */
       return(0L);

    s       =&p->f_param[0]; /* aktueller Schnittpkt in WK */
    s_lokal =&p->f_param[3]; /* aktueller Schnittpkt in WK */
    n       =&p->f_param[6];
   
    pt = (proctex_rec *)p->v_param[1];
    
    /* Von s den Wert der Kreiswelle berechnen */
    KREISW_HEIGHT(s_lokal,hell,pt->param[0],pt->param[1],pt->param[2]);
    /* Vom letzten Treffer den Wert der Kreiswelle berechnen */
    KREISW_HEIGHT(kreisw_last_s_l,hell1,pt->param[0],pt->param[1],pt->param[2]);

    /* Abstand aktueller minus letzter Schnittpunkt */
    hilf = n1[0]=s[0]-kreisw_last_s_g[0];
    f    =hilf*hilf;
    hilf = n1[1]=s[1]-kreisw_last_s_g[1];
    f   +=hilf*hilf;
    hilf = n1[2]=s[2]-kreisw_last_s_g[2];
    f   +=hilf*hilf;
    
    n1_len=(float)sqrt(f);
    f=1.0F/n1_len;
    n1[0] *= f;
    n1[1] *= f;
    n1[2] *= f;
    
    /* Jetzt Senkrechte zu n1,n bestimmen */
    vekprod(n,n1,n2);

    /* Von s aus senkrecht zu n, n1 um Abstand n1_len bewegen */
    s2[0] = n2[0]*n1_len+s[0];
    s2[1] = n2[1]*n1_len+s[1];
    s2[2] = n2[2]*n1_len+s[2];

    /* Davon lokale Koords berechnen */
    in.id=BMS_GET_LOKAL_PTEX_KOORD;
    in.v_param[0]=p->v_param[0];
    in.f_param[0]=s2[0];
    in.f_param[1]=s2[1];
    in.f_param[2]=s2[2];
    RPI_do(&in,&out);
    s2_tex[0]=out.f_param[0];
    s2_tex[1]=out.f_param[1];
    s2_tex[2]=out.f_param[2];

    KREISW_HEIGHT(s2_tex,hell2,pt->param[0],pt->param[1],pt->param[2]);
    
    m1=hell-hell1;
    m2=hell-hell2;
    vek_copy(n,e);
    f1=10.0F;
    proc_bump_n(m1*f1,m2*f1,n1,n2,e,n_neu);
    return(1);
}

/*****************************************************************************/
   
   long REFPROG 
RPI_message
   (PTR0 PLI_param *p,PTR1 PLI_param *p1)
{
   long ret=1;
   proctex_rec *px;
   char ss[200];
   PLI_param in,out;

   #ifdef AMIGA
   if (!libs_installed) return(0);
   #endif
  
   switch(p->id)
   {
      default: return(0L);  
      /* Alle anderen Messages interessieren uns nicht */
         
      case PLI_BEAMS_PTEX_INIT: 
                      px = (proctex_rec *)p->v_param[0];    
                      in.id=BMS_MESSAGE_WIN;
                      sprintf(ss,"Init: %s",px->plugin_name);
                      in.s_param[0]=ss;
                      RPI_do(&in,&out);                      
                      break;

      case PLI_BEAMS_PTEX_END: 
                      px = (proctex_rec *)p->v_param[0];    
                      break;
                      
      case FPLUG_INIT_MESSAGE: 
                      break;

      case FPLUG_CLOSE_MESSAGE: 
                      break;

      case PLI_CONNECT:
                      /* open amiga libraries */
                      #include <AmiOpen.c>
                      break;
                      
      case PLI_CLOSE_RESOURCES: 
                      /* close amiga libraries */
                      #include <AmiClose.c>
                      break;

      case PLI_BEAMS_PTEX_MATERIAL:
                      px = (proctex_rec *)p->v_param[1];
                      ret=0;
                      switch(px->intern_nr)
                      {
                        case 0: ret=lochbl(p); break;
                        case 1: ret=kreisw(p); break;
                      }
                      break;

      case PLI_BEAMS_PTEX_BUMP:
                      px = (proctex_rec *)p->v_param[1];    
                      ret=0;
                      switch(px->intern_nr)
                      {
                        case 1: ret=kreisw_bump(p,&p1->f_param[0]); break;
                      }
                      break;

      case PLI_BEAMS_PTEX_BACKGROUND:
                      break;
                      
   }
   return(ret);
}

/*****************************************************************************/
