/*****************************************************************************/
/*          refgateway stub functions for proper and save usage              */
/*                  of the RPI_do(in,out) commands.                          */
/*****************************************************************************/

/* title:    after-burner functions */
/* author:   Carsten Fuchs  */
/* version:  1.0, 30.6.99   */
/* updated:  16.11.99 by RG */
/* updated:  2.3.2000 by RG */

/*****************************************************************************/

#if MONZOOM_VERSION >= 45

/*****************************************************************************/
/*  allocate a new after-burner */

   RPIStub long
RPI_new_afterburner
   (RPI_HANDLE handle,char *name,long *id_nr)
{
   /* handle:      refgateway-handle
      id_nr:       destination for afterburner-id  
      returns:     0 if failed
                   1 if ok
   */
   PLI_param in,out;
   long ret;

   in.v_param[0]=handle;
   in.id=RPI_NEW_AFTER_BURNER;
   in.s_param[0]=name;
   if ((ret=RPI_do(&in,&out))!=0) {
       *id_nr=out.l_param[0];
   }
   return(ret);
}
/*****************************************************************************/
/*  delete an allocated after-burner */

   RPIStub long
RPI_del_afterburner
   (RPI_HANDLE handle,long id_nr)
{
   /* handle:      refgateway-handle
      id_nr:       afterburner-id  
      returns:     0 if failed
                   1 if ok
   */
   PLI_param in,out;

   in.v_param[0]=handle;
   in.l_param[0]=id_nr;
   in.id=RPI_DEL_AFTER_BURNER;
   return(RPI_do(&in,&out));
}
/*****************************************************************************/
/*  allocate a channel for an object */

   RPIStub long
RPI_afterburner_get_obj_kanal
   (RPI_HANDLE handle,long ab_id,RPI_OBJECT obj,long *id_nr)
{
   /* handle:      refgateway-handle
      ab_id:       id of the afterburner
      obj:         monzoom-object-handle
      id_nr:       destination for the id of the channel  
      returns:     0 if failed
                   1 if ok
   */
   PLI_param in,out;
   long ret;

   in.v_param[0]=handle;
   in.l_param[0]=ab_id;
   in.v_param[1]=obj;
   in.id=RPI_AFTER_BURNER_GET_OBJ_KANAL;
   if ((ret=RPI_do(&in,&out))!=0) {
       *id_nr=out.l_param[0];
   }
   return(ret);
}
/*****************************************************************************/
/*  release the channel-number for an object */

   RPIStub long
RPI_afterburner_release_obj_kanal
   (RPI_HANDLE handle,long ab_id,RPI_OBJECT obj)
{
   /* handle:      refgateway-handle
      ab_id:       id of the afterburner
      obj:         monzoom-object-handle 
      returns:     0 if failed
                   1 if ok
   */
   PLI_param in,out;

   in.v_param[0]=handle;
   in.v_param[1]=obj;
   in.l_param[0]=ab_id;
   in.id=RPI_AFTER_BURNER_RELEASE_OBJ_KANAL;
   return(RPI_do(&in,&out));
}
/*****************************************************************************/
/* get a map of all pixels, in which then kanal_id happens */
  
   RPIStub long
RPI_afterburner_get_kanal_map
   (RPI_HANDLE handle,long ab_id,long win_id,long kanal_id,unsigned char **map,long *npix)
{
   /* handle:      refgateway-handle
      ab_id:       id of the afterburner
      win_id:      number of the display
      kanal_id:    channel_id
      map:         destination for pointer to channel_map
      npix:        destination for the number of pixels set in the channel_map
      returns:     0 if failed
                   1 if ok
   */
   PLI_param in,out;
   long ret;

   in.v_param[0]=handle;
   in.l_param[0]=ab_id;
   in.l_param[1]=win_id;
   in.l_param[2]=kanal_id;
   in.id=RPI_AFTER_BURNER_GET_KANAL_MAP;
   in.v_param[1]=map;
   if ((ret=RPI_do(&in,&out))!=0) {
       *npix=out.l_param[0];
   }
   return(ret);
}
/*****************************************************************************/
/* this afterburner wants dephtbuffer-data  */

   RPIStub long 
RPI_afterburner_alloc_depthbuff
   (RPI_HANDLE handle,long ab_id)
{
   /* handle:      refgateway-handle
      ab_id:       id of the afterburner
      returns:     0 if failed
                   1 if ok
   */
   PLI_param in,out;

   in.v_param[0]=handle;
   in.l_param[0]=ab_id;
   in.id=RPI_AFTER_BURNER_ALLOC_DEPTHBUFF;
   return(RPI_do(&in,&out));
}
/*****************************************************************************/
/* this after-burner dont want the dephtbuffer-data any more */

   RPIStub long
RPI_afterburner_free_depthbuff
   (RPI_HANDLE handle,long ab_id)
{
   /* handle:      refgateway-handle
      ab_id:       id-nr of afterburner
      returns:     0 if failed
                   1 if ok
   */
   PLI_param in,out;

   in.v_param[0]=handle;
   in.l_param[0]=ab_id;
   in.id=RPI_AFTER_BURNER_FREE_DEPTHBUFF;
   return(RPI_do(&in,&out));
}
/*****************************************************************************/
/* get the depth-value of a pixel */

   RPIStub long
RPI_afterburner_get_pixel_depth
   (RPI_HANDLE handle,long ab_id,long win_id,long x,long y,float *depth)
{
   /* handle:      refgateway-handle
      ab_id:       id of the afterburner
      win_id:      number of the display
      x,y:         pixel-coordinates
      depth:       destination for depth-value of pixel
      returns:     0 if failed
                   1 if ok
   */
   PLI_param in,out;
   long ret;

   in.v_param[0]=handle;
   in.l_param[0]=ab_id;
   in.l_param[1]=win_id;
   in.l_param[2]=x;
   in.l_param[3]=y;
   in.id=RPI_AFTER_BURNER_GET_PIXEL_DEPTH;
   if ((ret=RPI_do(&in,&out))!=0) {
       *depth=out.f_param[0];
   }
   return(ret);
}
/*****************************************************************************/
/* get the depth-value of a set of pixels */

   RPIStub long
RPI_afterburner_get_multi_pixel_depth
   (RPI_HANDLE handle,long ab_id,long win_id,long npts,long *xy,float *depth)
{
   /* handle:      refgateway-handle
      ab_id:       id of the afterburner
      win_id:      number of the display
      npix:        number of pixels
      xy:          array with pixel-coordinates
      depth:       destination for depth-values of pixels
      returns:     0 if failed
                   1 if ok
   */
   PLI_param in,out;
   long ret;

   in.v_param[0]=handle;
   in.l_param[0]=ab_id;
   in.l_param[1]=win_id;
   in.l_param[2]=npts;
   in.v_param[1]=(void *)xy;
   in.v_param[2]=(void *)depth;
   in.id=RPI_AFTER_BURNER_GET_MULTI_PIXEL_DEPTH;
   if ((ret=RPI_do(&in,&out))!=0) {
       *depth=out.f_param[0];
   }
   return(ret);
}
/*****************************************************************************/
/* get the depth-values for a rectangle of pixels */
  
   RPIStub long
RPI_afterburner_get_pixel_rect_depth
   (RPI_HANDLE handle,long ab_id,long win_id,long x1,long y1,long x2,long y2,float *depth)
{
   /* handle:      refgateway-handle
      ab_id:       id of the afterburner
      win_id:      number of the display
      x1,y1,x2,y2: pixel-coordinates of rectangle
      depth:       destination for depth-values of pixels
      returns:     0 if failed
                   1 if ok
   */
   PLI_param in,out;

   in.v_param[0]=handle;
   in.l_param[0]=ab_id;
   in.l_param[1]=win_id;
   in.l_param[2]=x1;
   in.l_param[3]=y1;
   in.l_param[4]=x2;
   in.l_param[5]=y2;
   in.v_param[1]=(void *)depth;
   in.id=RPI_AFTER_BURNER_GET_PIXEL_RECT_DEPTH;
   return(RPI_do(&in,&out));
}
/*****************************************************************************/
/* get the depth-value of a background-pixel */
  
   RPIStub long
RPI_afterburner_get_background_depth
   (RPI_HANDLE handle,long ab_id,float *depth)
{
   /* handle:      refgateway-handle
      ab_id:       id of the afterburner
      depth:       destination for depth-value of pixel
      returns:     0 if failed
                   1 if ok
   */
   PLI_param in,out;
   long ret;

   in.v_param[0]=handle;
   in.l_param[0]=ab_id;
   in.id=RPI_AFTER_BURNER_GET_DEPTHBUFF_BACKGROUND_VALUE;
   if ((ret=RPI_do(&in,&out))!=0) {
       *depth=out.f_param[0];
   }
   return(ret);
}
/*****************************************************************************/

#endif /* if MONZOOM_VERSION >= 45 */

/*****************************************************************************/
