/*****************************************************************************/
/*          refgateway stub functions for proper and save usage              */
/*                  of the RPI_do(in,out) commands.                          */
/*****************************************************************************/

/* title:    display functions */
/* author:   Carsten Fuchs */
/* version:  1.0, 4.3.99 */
/* updated:  17.11.99 by RG */
/* updated:  24.11.99 by RG */
/* updated:  2.3.2000 by RG */

/*****************************************************************************/

#if MONZOOM_VERSION >= 45

/*****************************************************************************/
/* try to open a display */

   RPIStub long 
RPI_open_disp
   (RPI_HANDLE handle,char *title,long dx,long dy,unsigned long flags,long x,long y)
{
   /* handle:       refgateway-handle
      title:        title of window
      dx,dy:        desired display_size
      flags:        flags for the display
      x,y:          desired position for the window
      returns:      0 if failed
                    id-number of display if successful
   */
   PLI_param in,out;
   long ret;

   in.id=RPI_OPEN_DISP;
   in.v_param[0]=handle;
   in.s_param[0]=title;
   in.l_param[0]=dx;
   in.l_param[1]=dy;
   in.l_param[2]=(long)flags;
   in.l_param[4]=x;
   in.l_param[5]=y;
   if (0!=(ret=RPI_do(&in,&out)))
      ret=out.l_param[0];
   return(ret);
}
/*****************************************************************************/
/* try to close a display */

   RPIStub long 
RPI_close_disp
   (RPI_HANDLE handle,long disp_id)
{
   /* handle:       refgateway-handle
     disp_id:       id-number of display 
     returns:       0 if failed
                    1 if successful
   */
   PLI_param in,out;

   in.id=RPI_CLOSE_DISP;
   in.v_param[0]=handle;
   in.l_param[0]=disp_id;
   return(RPI_do(&in,&out));
}
/*****************************************************************************/
/* set the actual plot-color of a display */

   RPIStub long 
RPI_disp_set_col
   (RPI_HANDLE handle,long disp_id,long rgb[3])
{
   /* handle:       refgateway-handle
      disp_id:      id-number of display 
      rgb:          red/green/blue-component of color (0..255)
      returns:      0 if failed
                    1 if successful
   */
   PLI_param in,out;

   in.id=RPI_DISP_SET_COL;
   in.v_param[0]=handle;
   in.l_param[0]=disp_id;
   in.l_param[1]=rgb[0];
   in.l_param[2]=rgb[1];
   in.l_param[3]=rgb[2];
   return(RPI_do(&in,&out));
}
/*****************************************************************************/
/* get the actual-plot-color of a display */

   RPIStub long 
RPI_disp_get_col
   (RPI_HANDLE handle,long disp_id,long rgb[3])
{
   /* handle:       refgateway-handle
      disp_id:      id-number of display 
      rgb:          red/green/blue-component of color (0..255)
      returns:      0 if failed
                    1 if successful
   */
   PLI_param in,out;
   long ret;

   in.id=RPI_DISP_GET_COL;
   in.v_param[0]=handle;
   in.l_param[0]=disp_id;
   if ((ret=RPI_do(&in,&out))!=0) {
     rgb[0]=out.l_param[1];
     rgb[1]=out.l_param[2];
     rgb[2]=out.l_param[3];
   }
   return(ret);
}
/*****************************************************************************/
/* set a pixel of a display to the actual color */

   RPIStub long 
RPI_disp_set_pixel
   (RPI_HANDLE handle,long disp_id,long x,long y)
{
   /* handle:       refgateway-handle
      disp_id:      id-number of display
      x,y:          pixel-coordinates
      returns:      0 if failed
                    1 if successful
   */
   PLI_param in,out;

   in.id=RPI_DISP_SET_PIXEL;
   in.v_param[0]=handle;
   in.l_param[0]=disp_id;
   in.l_param[1]=x;
   in.l_param[2]=y;
   return(RPI_do(&in,&out));
}
/*****************************************************************************/
/* get the color of a pixel of a display */

   RPIStub long 
RPI_disp_get_pixel
   (RPI_HANDLE handle,long disp_id,long x,long y,long rgb[3])
{
   /* handle:       refgateway-handle
      disp_id:      id-number of display
      x,y:          pixel-coordinates
      rgb:          destination for color
      returns:      0 if failed
                    1 if successful
   */
   PLI_param in,out;
   long ret;

   in.id=RPI_DISP_GET_PIXEL;
   in.v_param[0]=handle;
   in.l_param[0]=disp_id;
   in.l_param[1]=x;
   in.l_param[2]=y;
   if ((ret=RPI_do(&in,&out))!=0) {
       rgb[0]=out.l_param[0];
       rgb[1]=out.l_param[1];
       rgb[2]=out.l_param[2];
   }
   return(ret);
}
/*****************************************************************************/
/* set an array of pixels of a display to the actual color */

   RPIStub long 
RPI_disp_set_multi_pixel
   (RPI_HANDLE handle,long disp_id,long npix,long *xy)
{
   /* handle:       refgateway-handle
      disp_id:      id-number of display
      npix:         number of pixels
      xy:           array with pixel-coordinates
      returns:      0 if failed
                    1 if successful
   */
   PLI_param in,out;

   in.id=RPI_DISP_SET_MULTI_PIXEL;
   in.v_param[0]=handle;
   in.l_param[0]=disp_id;
   in.l_param[1]=npix;
   in.v_param[1]=(void *)xy;
   return(RPI_do(&in,&out));
}
/*****************************************************************************/
/* set an array of pixels of a display to indiviual colors */

   RPIStub long 
RPI_disp_set_multi_color_pixel
   (RPI_HANDLE handle,long disp_id,long npix,long *xy,unsigned char *rgb)
{
   /* handle:       refgateway-handle
      disp_id:      id-number of display
      npix:         number of pixels
      xy:           array with pixel-coordinates
      rgb:          array with the color-values of each pixel
                    each pixel needs 3 (RGB) values (0..255)
      returns:      0 if failed
                    1 if successful
   */
   PLI_param in,out;

   in.id=RPI_DISP_SET_MULTI_COLOR_PIXEL;
   in.v_param[0]=handle;
   in.l_param[0]=disp_id;
   in.l_param[1]=npix;
   in.v_param[1]=(void *)xy;
   in.v_param[2]=(void *)rgb;
   return(RPI_do(&in,&out));
}
/*****************************************************************************/
/* get the color of an array of pixels of a display */

   RPIStub long 
RPI_disp_get_multi_pixel
   (RPI_HANDLE handle,long disp_id,long npix,long *xy,unsigned char *rgb)
{
   /* handle:       refgateway-handle
      disp_id:      id-number of display
      npix:         number of pixels
      xy:           array with pixel-coordinates
      rgb:          destination array for the color-values of each pixel
                    each pixel needs 3 (RGB) values (0..255)
      returns:      0 if failed
                    1 if successful
   */
   PLI_param in,out;

   in.id=RPI_DISP_GET_MULTI_PIXEL;
   in.v_param[0]=handle;
   in.l_param[0]=disp_id;
   in.l_param[1]=npix;
   in.v_param[1]=(void *)xy;
   in.v_param[2]=(void *)rgb;
   return(RPI_do(&in,&out));
}
/*****************************************************************************/
/* move the drawing pen to a new position */

   RPIStub long 
RPI_disp_moveto
   (RPI_HANDLE handle,long disp_id,long x,long y)
{
   /* handle:       refgateway-handle
      disp_id       id-number of display
      x,y           new coordinates
      returns:      0 if failed
                    1 if successful
   */
   PLI_param in,out;

   in.id=RPI_DISP_MOVETO;
   in.v_param[0]=handle;
   in.l_param[0]=disp_id;
   in.l_param[1]=x;
   in.l_param[2]=y;
   return(RPI_do(&in,&out));
}
/*****************************************************************************/
/* draw a line from the actual position of the drawing pen to a new position */

   RPIStub long 
RPI_disp_drawto
   (RPI_HANDLE handle,long disp_id,long x,long y)
{
   /* handle:       refgateway-handle
      disp_id:      id-number of display
      x,y:          new coordinates
      returns:      0 if failed
                    1 if successful
   */
   PLI_param in,out;

   in.id=RPI_DISP_DRAWTO;
   in.v_param[0]=handle;
   in.l_param[0]=disp_id;
   in.l_param[1]=x;
   in.l_param[2]=y;
   return(RPI_do(&in,&out));
}
/*****************************************************************************/
/* draws a polyline in a display with the actual color */

   RPIStub long 
RPI_disp_polyline
   (RPI_HANDLE handle,long disp_id,long npix,long *xy)
{
   /* handle:       refgateway-handle
      disp_id:      id-number of display
      npix:         number of points
      xy:           array with pixel-coordinates
      returns:      0 if failed
                    1 if successful
   */
   PLI_param in,out;

   in.id=RPI_DISP_POLYLINE;
   in.v_param[0]=handle;
   in.l_param[0]=disp_id;
   in.l_param[1]=npix;
   in.v_param[1]=(void *)xy;
   return(RPI_do(&in,&out));
}
/*****************************************************************************/
/* draws a set of line-segments in a display with the actual color */

   RPIStub long 
RPI_disp_polysegment
   (RPI_HANDLE handle,long disp_id,long nlines,long *xyxy)
{
   /* handle:       refgateway-handle
      disp_id:      id-number of display
      nlines:       number of lines
      xyxy:         array with pixel-coordinates
      returns:      0 if failed
                    1 if successful
   */
   PLI_param in,out;

   in.id=RPI_DISP_POLYSEGMENT;
   in.v_param[0]=handle;
   in.l_param[0]=disp_id;
   in.l_param[1]=nlines;
   in.v_param[1]=(void *)xyxy;
   return(RPI_do(&in,&out));
}
/*****************************************************************************/
/* draws a set of triangles in a display with the actual color */

   RPIStub long 
RPI_disp_polytriangles
   (RPI_HANDLE handle,long disp_id,long ntriangles,long *xyxyxy)
{
   /* handle:       refgateway-handle
      disp_id:      id-number of display
      ntriangles:   number of triangles
      xyxyxy:       array with vertex-coordinates of triangles
      returns:      0 if failed
                    1 if successful
   */
   PLI_param in,out;

   in.id=RPI_DISP_POLYTRIANGLES;
   in.v_param[0]=handle;
   in.l_param[0]=disp_id;
   in.l_param[1]=ntriangles;
   in.v_param[1]=(void *)xyxyxy;
   return(RPI_do(&in,&out));
}
/*****************************************************************************/
/* fill a rectangle of a display with the actual color */

   RPIStub long 
RPI_disp_fill_rect
   (RPI_HANDLE handle,long disp_id,long x1,long y1,long x2,long y2)
{
   /* handle:       refgateway-handle
      disp_id:      id-number of display
      x1,y1,x2,y2:  rectangle coordinates
      returns:      0 if failed
                    1 if successful
   */
   PLI_param in,out;

   in.id=RPI_DISP_FILLRECT;
   in.v_param[0]=handle;
   in.l_param[0]=disp_id;
   in.l_param[1]=x1;
   in.l_param[2]=y1;
   in.l_param[3]=x2;
   in.l_param[4]=y2;
   return(RPI_do(&in,&out));
}
/*****************************************************************************/
/* set the colors of a rectangle of pixels */

   RPIStub long 
RPI_disp_set_rect
   (RPI_HANDLE handle,long disp_id,long x1,long y1,long x2,long y2,unsigned char *rgb)
{
   /* handle:       refgateway-handle
      disp_id:      id-number of display
      x1,y1,x2,y2:  rectangle coordinates
      rgb:          rgb-values of pixels (at least (x2-x1+1)*(y2-y1+1)*3 bytes)
      returns:      0 if failed
                    1 if successful
   */
   PLI_param in,out;

   in.id=RPI_DISP_SETRECT;
   in.v_param[0]=handle;
   in.l_param[0]=disp_id;
   in.l_param[1]=x1;
   in.l_param[2]=y1;
   in.l_param[3]=x2;
   in.l_param[4]=y2;
   in.v_param[1]=(void *)rgb;
   return(RPI_do(&in,&out));
}
/*****************************************************************************/
/* set the colors of a rectangle of pixels with alpha channel */

   RPIStub long 
RPI_disp_set_rect_alpha
   (RPI_HANDLE handle,long disp_id,long x1,long y1,long x2,long y2,unsigned char *rgb,unsigned char *alpha)
{
   /* handle:       refgateway-handle
      disp_id:      id-number of display
      x1,y1,x2,y2:  rectangle coordinates
      rgb:          rgb-values of pixels (at least (x2-x1+1)*(y2-y1+1)*3 bytes)
      alpha:        alpha-values of pixels (at least (x2-x1+1)*(y2-y1+1) bytes)
      returns:      0 if failed
                    1 if successful
   */
   PLI_param in,out;

   in.id=RPI_DISP_SETRECT_ALPHA;
   in.v_param[0]=handle;
   in.l_param[0]=disp_id;
   in.l_param[1]=x1;
   in.l_param[2]=y1;
   in.l_param[3]=x2;
   in.l_param[4]=y2;
   in.v_param[1]=(void *)rgb;
   in.v_param[2]=(void *)alpha;
   return(RPI_do(&in,&out));
}
/*****************************************************************************/
/* copy a rectangle of pixels from one display to another
   combined with an alpha channel
*/
   RPIStub long 
RPI_disp_copy_rect_alpha_to
   (RPI_HANDLE handle,long disp_from_id,long disp_to_id,
    long src_x1,long src_y1,long src_x2,long src_y2,
    long dest_x,long dest_y,unsigned char *alpha)
{
   /* handle:        refgateway-handle
      disp_from_id:  id-number of source display
      disp_to_id:    id-number of destination display
      scr_x1,scr_y1: left/top coordinates of source rectangle
      scr_x2,scr_y2: right/bottom coordinates of source rectangle
      x,y:           left/top coordinates in destination display 
      alpha:         alpha-values of pixels (at least (x2-x1+1)*(y2-y1+1) bytes)
      returns:       0 if failed
                     1 if successful
   */
   PLI_param in,out;

   in.id=RPI_DISP_COPY_RECT_ALPHA;
   in.v_param[0]=handle;
   in.l_param[0]=disp_from_id;
   in.l_param[1]=disp_to_id;
   in.l_param[2]=src_x1;
   in.l_param[3]=src_y1;
   in.l_param[4]=src_x2;
   in.l_param[5]=src_y2;
   in.l_param[6]=dest_x;
   in.l_param[7]=dest_y;
   in.v_param[1]=(void *)alpha;
   return(RPI_do(&in,&out));
}
/*****************************************************************************/
/* get the colors of a rectangle of pixels */

   RPIStub long 
RPI_disp_get_rect
   (RPI_HANDLE handle,long disp_id,long x1,long y1,long x2,long y2,unsigned char *rgb)
{
   /* handle:       refgateway-handle
      disp_id:      id-number of display
      x1,y1,x2,y2:  rectangle coordinates
      rgb:          rgb-values of pixels (at least (x2-x1+1)*(y2-y1+1)*3 bytes)
      returns:      0 if failed
                    1 if successful
   */
   PLI_param in,out;

   in.id=RPI_DISP_GETRECT;
   in.v_param[0]=handle;
   in.l_param[0]=disp_id;
   in.l_param[1]=x1;
   in.l_param[2]=y1;
   in.l_param[3]=x2;
   in.l_param[4]=y2;
   in.v_param[1]=(void *)rgb;
   return(RPI_do(&in,&out));
}
/*****************************************************************************/
/* execute all stored operations of a display */

   RPIStub long 
RPI_disp_buffer_flush
   (RPI_HANDLE handle,long disp_id)
{
   /* handle:       refgateway-handle
      disp_id:      id-number of display
      returns:      0 if failed
                    1 if successful
   */
   PLI_param in,out;

   in.id=RPI_DISP_BUFFER_FLUSH;
   in.v_param[0]=handle;
   in.l_param[0]=disp_id;
   return(RPI_do(&in,&out));
}
/*****************************************************************************/
/* copy the contents of a display to its window */

   RPIStub long 
RPI_disp_refresh_window
   (RPI_HANDLE handle,long disp_id)
{
   /* handle:       refgateway-handle
      disp_id:      id-number of display
      returns:      0 if failed
                    1 if successful
   */
   PLI_param in,out;

   in.id=RPI_DISP_REFRESH_WINDOW;
   in.v_param[0]=handle;
   in.l_param[0]=disp_id;
   return(RPI_do(&in,&out));
}
/*****************************************************************************/
/* copy the contents of a display inside clipping rectangle to its window */
    
   RPIStub long 
RPI_disp_refresh_window_clip
   (RPI_HANDLE handle,long disp_id)
{
   /* handle:       refgateway-handle
      disp_id:      id-number of display
      returns:      0 if failed
                    1 if successful
   */
   PLI_param in,out;

   in.id=RPI_DISP_REFRESH_WINDOW_CLIP;
   in.v_param[0]=handle;
   in.l_param[0]=disp_id;
   return(RPI_do(&in,&out));
}
/*****************************************************************************/
/* set the actual clipping-rectangle of a display */

   RPIStub long 
RPI_disp_set_clip
   (RPI_HANDLE handle,long disp_id,long x1,long y1,long x2,long y2)
{
   /* handle:       refgateway-handle
      disp_id:      id-number of display
      x1,y1,x2,y2:  clipping-rectangle
      returns:      0 if failed
                    1 if successful
   */
   PLI_param in,out;
 
   in.id=RPI_DISP_SET_CLIP;
   in.v_param[0]=handle;
   in.l_param[0]=disp_id;
   in.l_param[1]=x1;
   in.l_param[2]=y1;
   in.l_param[3]=x2;
   in.l_param[4]=y2;
   return(RPI_do(&in,&out));
}
/*****************************************************************************/
/* get the actual clipping-rectangle of a display */

   RPIStub long 
RPI_disp_get_clip
   (RPI_HANDLE handle,long disp_id,long *x1,long *y1,long *x2,long *y2)
{
   /* handle:       refgateway-handle
      disp_id:      id-number of display
      x1,y1,x2,y2:  clipping-rectangle
      returns:      0 if failed
                    1 if successful
   */
   PLI_param in,out;
   long ret;

   in.id=RPI_DISP_GET_CLIP;
   in.v_param[0]=handle;
   in.l_param[0]=disp_id;
   if ((ret=RPI_do(&in,&out))!=0) {
      *x1=out.l_param[0];
      *y1=out.l_param[1];
      *x2=out.l_param[2];
      *y2=out.l_param[3];
   }
   return(ret);
}
/*****************************************************************************/
/* set the actual clipping-rectangle to the full size of the display */

   RPIStub long 
RPI_disp_full_clip
   (RPI_HANDLE handle,long disp_id)
{
   /* handle:       refgateway-handle
      disp_id:      id-number of display
      returns:      0 if failed
                    1 if successful
   */
   PLI_param in,out;

   in.id=RPI_DISP_FULL_CLIP;
   in.v_param[0]=handle;
   in.l_param[0]=disp_id;
   return(RPI_do(&in,&out));
}
/*****************************************************************************/
/* get the actual size of a display */

   RPIStub long 
RPI_disp_get_size
   (RPI_HANDLE handle,long disp_id,long *dx,long *dy)
{
   /* handle:       refgateway-handle
      disp_id:      id-number of display
      dx:           x-pixels
      dy:           y-pixels
      returns:      0 if failed
                    1 if successful
   */
   PLI_param in,out;
   long ret;

   in.id=RPI_DISP_GET_SIZE;
   in.v_param[0]=handle;
   in.l_param[0]=disp_id;
   if ((ret=RPI_do(&in,&out))!=0) {
      *dx=out.l_param[0];
      *dy=out.l_param[1];
   }
   return(ret);
}
/*****************************************************************************/
/* ask for the rgb-format of a display */
   
   RPIStub long 
RPI_disp_get_rgb_mode
   (RPI_HANDLE handle,long disp_id,long *mode)
{
   /* handle:       refgateway-handle
      disp_id:      id-number of display
      returns:      0 if failed
                    1 if successful
   */
   PLI_param in,out;
   long ret;

   in.id=RPI_DISP_GET_RGB_MODE;
   in.v_param[0]=handle;
   in.l_param[0]=disp_id;
   if ((ret=RPI_do(&in,&out))!=0)
      *mode=out.l_param[0];
   return(ret);
}
/*****************************************************************************/
/* ask for the rgb-buffer of a display */
   
   RPIStub long 
RPI_disp_get_rgb_buffer
   (RPI_HANDLE handle,long disp_id,unsigned char **buffer)
{
   /* handle:       refgateway-handle
      disp_id:      id-number of display
      returns:      0 if failed
                    1 if successful
   */
   PLI_param in,out;
   long ret;

   in.id=RPI_DISP_GET_RGB_BUFFER;
   in.v_param[0]=handle;
   in.l_param[0]=disp_id;
   if ((ret=RPI_do(&in,&out))!=0)
      *buffer=(unsigned char *)out.v_param[0];
   return(ret);
}
/*****************************************************************************/
/* ask for the pixel-size for a text-string on a display */
   
   RPIStub long 
RPI_disp_get_text_size
   (RPI_HANDLE handle,long disp_id,char *text,long *dx,long *dy)
{
   /* handle:       refgateway-handle
      disp_id:      id-number of display
      text:         text-string
      dx,dy:        size of <text> in pixels
      returns:      0 if failed
                    1 if successful
   */
   PLI_param in,out;
   long ret;

   in.id=RPI_DISP_GET_TEXT_SIZE;
   in.v_param[0]=handle;
   in.s_param[0]=text;
   in.l_param[0]=disp_id;
   if ((ret=RPI_do(&in,&out))!=0) {
      *dx=out.l_param[0];
      *dy=out.l_param[1];
   }
   return(ret);
}
/*****************************************************************************/
/* set the font type (size) for a display. for size defines see rgw_defs.h */

   RPIStub long 
RPI_disp_set_font
   (RPI_HANDLE handle,long disp_id,long type)
{
   /* handle:       refgateway-handle
      disp_id:      id-number of display
      type:         0=system font, or 1..4 for small to extra large font
      returns:      0 if failed
                    1 if successful
   */
   PLI_param in,out;

   in.id=RPI_DISP_SET_FONT;
   in.v_param[0]=handle;
   in.l_param[0]=disp_id;
   in.l_param[1]=type;
   return(RPI_do(&in,&out));
}
/*****************************************************************************/
/* plot a text-string on a display */
   
   RPIStub long 
RPI_disp_text
   (RPI_HANDLE handle,long disp_id,char *text,long x,long y)
{
   /* handle:       refgateway-handle
      disp_id:      id-number of display
      text:         text-string
      x,y:          position of <text> in pixels
      returns:      0 if failed
                    1 if successful
   */
   PLI_param in,out;

   in.id=RPI_DISP_TEXT;
   in.v_param[0]=handle;
   in.l_param[0]=disp_id;
   in.s_param[0]=text;
   in.l_param[1]=x;
   in.l_param[2]=y;
   return(RPI_do(&in,&out));
}
/*****************************************************************************/
/* plot a text-string on a display, defining the font characteristics */
   
   RPIStub long 
RPI_disp_text1
   (RPI_HANDLE handle,long disp_id,char *text,long x,long y,unsigned long flags)
{
   /* handle:       refgateway-handle
      disp_id:      id-number of display
      text:         text-string
      x,y:          position of <text> in pixels
      flags:        normal(0), italic (1), bold(2) or italic & bold (3)
      returns:      0 if failed
                    1 if successful
   */
   PLI_param in,out;

   in.id=RPI_DISP_TEXT1;
   in.v_param[0]=handle;
   in.l_param[0]=disp_id;
   in.s_param[0]=text;
   in.l_param[1]=x;
   in.l_param[2]=y;
   in.l_param[3]=flags;
   return(RPI_do(&in,&out));
}
/*****************************************************************************/
/* Try to resize a display*/

   RPIStub long 
RPI_resize_disp
   (RPI_HANDLE handle,long disp_id,long dx,long dy)
{
   /* handle:       refgateway-handle
      disp_id       id-number of display 
      dx,dy         desired display_size
      returns:      0 if failed
                    id-number of display if successful
   */
   PLI_param in,out;

   in.id=RPI_RESIZE_DISP;
   in.v_param[0]=handle;
   in.l_param[0]=disp_id;
   in.l_param[1]=dx;
   in.l_param[2]=dy;
   return(RPI_do(&in,&out));
}
/*****************************************************************************/
/* try to open a display and load an picture into it,
   the size of the picture is the size of the display*/

   RPIStub long 
RPI_load_pic_get_disp
   (RPI_HANDLE handle,char *pic_file,unsigned long flags,long x,long y)
{
   /* handle:       refgateway-handle
      pic_file:     name of picture
      flags:        flags for the display
      x,y:          desired position for the window
      returns:      0 if failed
                    id-number of display if successful
   */
   PLI_param in,out;
   long ret;

   in.id=RPI_LOAD_PIC_GET_DISP;
   in.v_param[0]=handle;
   in.s_param[0]=pic_file;
   in.l_param[0]=(long)flags;
   in.l_param[1]=x;
   in.l_param[2]=y;
   if (0!=(ret=RPI_do(&in,&out)))
      ret=out.l_param[0];
   return(ret);
}
/*****************************************************************************/
/* Try to get the projection of a 3D-Point on a display*/

   RPIStub long 
RPI_disp_project_pt
   (RPI_HANDLE handle,long disp_id,float pt[3],long *x,long *y)
{
   /* handle:       refgateway-handle
      disp_id       id-number of display 
      pt            3D-Koordinates of Point
      x,y           destinations for the pixel-koordintes of th
                    projection of the 3D-point on the display
      returns:      0 if failed
                    1 if successful
   */
   PLI_param in,out;
   long ret;

   in.id=RPI_DISP_PROJECT_PT;
   in.v_param[0]=handle;
   in.l_param[0]=disp_id;
   in.f_param[0]=pt[0];
   in.f_param[1]=pt[1];
   in.f_param[2]=pt[2];
   if (0!=(ret=RPI_do(&in,&out))) {
      *x=out.l_param[0];
      *y=out.l_param[1];
   }
   return(ret);
}
/*****************************************************************************/
/* Try to get the actual camera, which defines the projection of a display */

   RPIStub long 
RPI_disp_get_camera
   (RPI_HANDLE handle,long disp_id,RPI_OBJECT *kam)
{
   /* handle:       refgateway-handle
      disp_id       id-number of display 
      kam:          destination for camera of the display
                    if one is connected, otherwise 0
      returns:      0 if failed
                    1 if successful
   */
   PLI_param in,out;
   long ret;

   in.id=RPI_DISP_GET_CAMERA;
   in.v_param[0]=handle;
   in.l_param[0]=disp_id;
   if (0!=(ret=RPI_do(&in,&out)))
      *kam=(RPI_OBJECT *)out.v_param[0];
   return(ret);
}
/*****************************************************************************/
/* ask for the actual position of a display */

   RPIStub long 
RPI_disp_get_pos
   (RPI_HANDLE handle,long disp_id,long *x,long *y)
{
   /* handle:       refgateway-handle
     disp_id:       id-number of display 
     *x,*y:         destination for window position
     returns:       0 if failed
                    1 if successful
   */
   PLI_param in,out;
   long ret;

   in.id=RPI_DISP_GET_POS;
   in.v_param[0]=handle;
   in.l_param[0]=disp_id;
   if (0!=(ret=RPI_do(&in,&out))) {
       *x=out.l_param[0];
       *y=out.l_param[1];
   }
   return(ret);
}
/*****************************************************************************/
/* determine if a key is used for hotkey-purposes by monzoom.
   if not, the plugin can be shure to get a PLI_DISP_KEY_PRESSED-
   message when the user presses this key, when the display-window
   is active and the key is pressed.
*/

   RPIStub long 
RPI_disp_is_key_used
   (RPI_HANDLE handle,long disp_id,unsigned char key,long *used)
{
   /* handle:       refgateway-handle
      disp_id       id-number of display 
      key:          key char
      returns:      0 if failed
                    1 if successfull
   */
   PLI_param in,out;
   long ret;

   *used=0;
   in.id=RPI_DISP_IS_KEY_USED;
   in.v_param[0]=handle;
   in.l_param[0]=disp_id;
   in.l_param[1]=(long)key;
   if (0!=(ret=RPI_do(&in,&out)))
      *used=out.l_param[0];
   return(ret);
}
/*****************************************************************************/

#endif /* if MONZOOM_VERSION >= 45 */

/*****************************************************************************/
