/*****************************************************************************/
/*          refgateway stub functions for proper and save usage              */
/*                  of the RPI_do(in,out) commands.                          */
/*****************************************************************************/

/* title:    geo-object functions */
/* author:   Carsten Fuchs */
/* version:  1.0, 16.9.97 */
/* updated:  23.3.98 by RG */
/* version:  1.1, 6.9.99 */
/* updated:  17.11.99 by RG */
/* updated:  2.3.2000 by RG */

/*****************************************************************************/
/* set the coordinates of a point */

   RPIStub long
RPI_geo_set_pt
   (RPI_HANDLE handle,RPI_OBJECT obj,long pointnr,float v[3])
{
   /* handle:       refgateway-handle
      obj:          object-handle
      pointnr:      index number of point
      v:            new x,y and z coordinates of point
      returns:      0 if failed
                    1 if ok
   */
   PLI_param in,out;
   long i;

   in.v_param[0]=handle;
   in.id=RPI_GEO_SET_PT;
   in.v_param[1]=obj;
   in.l_param[0]=pointnr;
   for (i=0;i<3;i++)
      in.f_param[i]=v[i];
   return(RPI_do(&in,&out));
}
/*****************************************************************************/
/* get the coordinates of a point */

   RPIStub long
RPI_geo_get_pt
   (RPI_HANDLE handle,RPI_OBJECT obj,long pointnr,float v[3])
{
   /* handle:       refgateway-handle
      obj:          object-handle
      pointnr:      index number of point
      v:            destination for x,y and z coordinates of point
      returns:      0 if failed
                    1 if ok
   */
   PLI_param in,out;
   long i,ret;

   in.v_param[0]=handle;
   in.id=RPI_GEO_GET_PT;
   in.v_param[1]=obj;
   in.l_param[0]=pointnr;
   if (0!=(ret=RPI_do(&in,&out))) {
      for (i=0;i<3;i++) 
         v[i]=out.f_param[i];
   }
   return(ret);
}
/*****************************************************************************/
/* set the coordinates of a number of points */

   RPIStub long
RPI_geo_set_pts
   (RPI_HANDLE handle,RPI_OBJECT obj,long npts,long *pt_indices,float *koords_xyz)
{
   /* handle:       refgateway-handle
      obj:          object-handle
      npts:         number of points to be changed
      pt_indices:   if pt_indices==0 the coordinates of all points from 0 to npts-1
                    will be set, else pt_indices holds the indices of npts points
                    and only the given points are changed.
      koords_xyz:   the coordinates of npts points in the format xyzxyzxyz...
      returns:      0 if failed
                    1 if ok
   */
   PLI_param in,out;

   in.v_param[0]=handle;
   in.id=RPI_GEO_SET_PTS;
   in.v_param[1]=obj;
   in.l_param[0]=npts;
   in.v_param[2]=(long *)pt_indices;
   in.v_param[3]=(float *)koords_xyz;
   return(RPI_do(&in,&out));
}
/*****************************************************************************/
/* get the coordinates of a number of points */

   RPIStub long
RPI_geo_get_pts
   (RPI_HANDLE handle,RPI_OBJECT obj,long npts,long *pt_indices,float *koords_xyz)
{
   /* handle:       refgateway-handle
      obj:          object-handle
      npts:         number of points to be readed
      pt_indices:   if pt_indices==0 the coordinates of all points from 0 to npts-1
                    will be returned, else pt_indices holds the indices of npts points
                    and only the coordinates of the given points are returned. 
      koords_xyz:   destination for the coordinates of npts points in the format xyzxyzxyz...
      returns:      0 if failed
                    1 if ok
   */
   PLI_param in,out;

   in.v_param[0]=handle;
   in.id=RPI_GEO_GET_PTS;
   in.v_param[1]=obj;
   in.l_param[0]=npts;
   in.v_param[2]=(long *)pt_indices;
   in.v_param[3]=(float *)koords_xyz;
   return(RPI_do(&in,&out));
}
/*****************************************************************************/
/* add a new point to an object */

   RPIStub long
RPI_geo_new_pt
   (RPI_HANDLE handle,RPI_OBJECT obj,float v[3],long *nr)
{
   /* handle:       refgateway-handle
      obj:          object-handle
      v:            x,y and z coordinates of new point
      nr:           destination for index number of new point
      returns:      0 if failed
                    1 if ok
   */
   PLI_param in,out;
   long i,ret;

   in.v_param[0]=handle;
   in.id=RPI_GEO_NEW_PT;
   in.v_param[1]=obj;
   for (i=0;i<3;i++)
      in.f_param[i]=v[i];
   if (0!=(ret=RPI_do(&in,&out)))
     *nr=out.l_param[0];
   return(ret);     
}
/*****************************************************************************/
/* insert a new point before a specified point to an object */

   RPIStub long
RPI_geo_insert_pt
   (RPI_HANDLE handle,RPI_OBJECT obj,long nr,float v[3])
{
   /* handle:       refgateway-handle
      obj:          object-handle
      nr:           index number of new point
      v:            x,y and z coordinates of new point
      returns:      0 if failed
                    1 if ok
   */
   PLI_param in,out;
   long i;

   in.v_param[0]=handle;
   in.id=RPI_GEO_INSERT_PT;
   in.v_param[1]=obj;
   in.l_param[0]=nr;
   for (i=0;i<3;i++)
      in.f_param[i]=v[i];
   return(RPI_do(&in,&out));     
}
/*****************************************************************************/
/* delete a point from an object */

   RPIStub long
RPI_geo_delete_pt
   (RPI_HANDLE handle,RPI_OBJECT obj,long pointnr)
{
   /* handle:       refgateway-handle
      obj:          object-handle
      pointnr:      index number of point to delete
      returns:      0 if failed
                    1 if ok
   */
   PLI_param in,out;

   in.v_param[0]=handle;
   in.id=RPI_GEO_DELETE_PT;
   in.v_param[1]=obj;
   in.l_param[0]=pointnr;
   return(RPI_do(&in,&out));     
}
/*****************************************************************************/
/* delete all selected points from an object */

   RPIStub long
RPI_geo_delete_selected_pts
   (RPI_HANDLE handle,RPI_OBJECT obj)
{
   /* handle:       refgateway-handle
      obj:          object-handle
      returns:      0 if failed
                    1 if ok
   */
   PLI_param in,out;

   in.v_param[0]=handle;
   in.id=RPI_GEO_DELETE_SELECTED_PTS;
   in.v_param[1]=obj;   
   return(RPI_do(&in,&out));     
}
/*****************************************************************************/
/* get number of selected points of an object */

   RPIStub long
RPI_geo_n_selected_pts
   (RPI_HANDLE handle,RPI_OBJECT obj,long *n_selected)
{
   /* handle:       refgateway-handle
      obj:          object-handle
      n_selected:   destination for the number of selected points
      returns:      0 if failed
                    1 if ok
   */
   PLI_param in,out;
   long ret;

   in.v_param[0]=handle;
   in.id=RPI_GEO_N_SELECTED_PTS;
   in.v_param[1]=obj;
   if (0!=(ret=RPI_do(&in,&out)))
      *n_selected=out.l_param[0];
   return(ret);       
}
/*****************************************************************************/
/* get the index numbers of the selected points of an object */
   
   RPIStub long
RPI_geo_get_selected_pts
   (RPI_HANDLE handle,RPI_OBJECT obj,long *array,long maxarray)
{
   /* handle:       refgateway-handle
      obj:          object-handle
      array:        destination to store index numbers of selected points
      maxarray:     maximal items that can be stored in array
      returns:      0 if failed
                    1 if ok
   */
   PLI_param in,out;

   in.v_param[0]=handle;
   in.id=RPI_GEO_GET_SELECTED_PTS;
   in.v_param[1]=obj;
   in.v_param[2]=(void *)array;
   in.l_param[0]=maxarray;
   return(RPI_do(&in,&out));       
}
/*****************************************************************************/
/* get the select state of a point */

   RPIStub long
RPI_geo_pt_is_selected
   (RPI_HANDLE handle,RPI_OBJECT obj,long pointnr,long *is_selected)
{             
   /* handle:       refgateway-handle
      obj:          object-handle
      pointnr:      index number of point to test
      is_selected:  destination for the select state, 1=selected
      returns:      0 if failed
                    1 if ok
   */
   PLI_param in,out;
   long ret;

   in.v_param[0]=handle;
   in.id=RPI_GEO_PT_IS_SELECTED;
   in.v_param[1]=obj;
   in.l_param[0]=pointnr;
   if (0!=(ret=RPI_do(&in,&out)))
      *is_selected=out.l_param[0];
   return(ret);       
}
/*****************************************************************************/
/* select a point */

   RPIStub long
RPI_geo_select_pt
   (RPI_HANDLE handle,RPI_OBJECT obj,long pointnr)
{             
   /* handle:       refgateway-handle
      obj:          object-handle
      pointnr:      index number of point to select
      returns:      0 if failed
                    1 if ok
   */
   PLI_param in,out;

   in.v_param[0]=handle;
   in.id=RPI_GEO_SELECT_PT;
   in.v_param[1]=obj;
   in.l_param[0]=pointnr;
   return(RPI_do(&in,&out));       
}
/*****************************************************************************/
/* unselect point */

   RPIStub long
RPI_geo_unselect_pt
   (RPI_HANDLE handle,RPI_OBJECT obj,long pointnr)
{             
   /* handle:       refgateway-handle
      obj:          object-handle
      pointnr:      index number of point to unselect
      returns:      0 if failed
                    1 if ok
   */
   PLI_param in,out;

   in.v_param[0]=handle;
   in.id=RPI_GEO_UNSELECT_PT;
   in.v_param[1]=obj;
   in.l_param[0]=pointnr;
   return(RPI_do(&in,&out));       
}
/*****************************************************************************/
/* select all points of an object */

   RPIStub long
RPI_geo_select_all_pts
   (RPI_HANDLE handle,RPI_OBJECT obj)
{             
   /* handle:       refgateway-handle
      obj:          object-handle
      returns:      0 if failed
                    1 if ok
   */
   PLI_param in,out;

   in.v_param[0]=handle;
   in.id=RPI_GEO_SELECT_ALL_PTS;
   in.v_param[1]=obj;
   return(RPI_do(&in,&out));       
}
/*****************************************************************************/
/* unselect all points of an object */

   RPIStub long
RPI_geo_unselect_all_pts
   (RPI_HANDLE handle,RPI_OBJECT obj)
{             
   /* handle:       refgateway-handle
      obj:          object-handle
      returns:      0 if failed
                    1 if ok
   */
   PLI_param in,out;

   in.v_param[0]=handle;
   in.id=RPI_GEO_UNSELECT_ALL_PTS;
   in.v_param[1]=obj;
   return(RPI_do(&in,&out));       
}
/*****************************************************************************/
/* move all selected points relative to current position */

   RPIStub long
RPI_geo_move_selected_pts
   (RPI_HANDLE handle,RPI_OBJECT obj,float delta[3])
{             
   /* handle:       refgateway-handle
      obj:          object-handle
      delta:        dx,dy,dz local coords to move all selected pts
      returns:      0 if failed
                    1 if ok
   */
   PLI_param in,out;
   long i;

   in.v_param[0]=handle;
   in.id=RPI_GEO_MOVE_SELECTED_PTS;
   in.v_param[1]=obj;
   for (i=0;i<3;i++)
      in.f_param[i]=delta[i];
   return(RPI_do(&in,&out));       
}
/*****************************************************************************/
/* rotate all selected points around a specified center */

   RPIStub long
RPI_geo_rot_selected_pts
   (RPI_HANDLE handle,RPI_OBJECT obj,float angle,float mitpt[3],long axis)
{             
   /* handle:       refgateway-handle
      obj:          object-handle
      angle:        rotating angle in degrees
      mitpt:        center (local coords) to rotate all selected points around
      axis:         local axis to rotate around (0=x,1=y,2=z)
      returns:      0 if failed
                    1 if ok
   */
   PLI_param in,out;
   long i;

   in.v_param[0]=handle;
   in.id=RPI_GEO_ROT_SELECTED_PTS;
   in.v_param[1]=obj;
   in.f_param[0]=angle;
   for (i=0;i<3;i++)
      in.f_param[i+1]=mitpt[i];
   in.l_param[0]=axis;
   return(RPI_do(&in,&out));       
}
/*****************************************************************************/

   RPIStub long
RPI_geo_size_selected_pts
   (RPI_HANDLE handle,RPI_OBJECT obj,float size[3])
{             
   /* handle:       refgateway-handle
      obj:          object-handle
      size:         sx,sy,sz sizing-factors for x,y,z-direction
      returns:      0 if failed
                    1 if ok
   */
   PLI_param in,out;
   long i;

   in.v_param[0]=handle;
   in.id=RPI_GEO_SIZE_SELECTED_PTS;
   in.v_param[1]=obj;
   for (i=0;i<3;i++)
      in.f_param[i]=size[i];
   return(RPI_do(&in,&out));       
}
/*****************************************************************************/
/* get the number of points of an object */

   RPIStub long
RPI_geo_npts
   (RPI_HANDLE handle,RPI_OBJECT obj,long *npoints)
{             
   /* handle:       refgateway-handle
      obj:          object-handle
      npoints:      destination for the number of points
      returns:      0 if failed
                    1 if ok
   */
   PLI_param in,out;
   long ret;

   in.v_param[0]=handle;
   in.id=RPI_GEO_NPTS;
   in.v_param[1]=obj;
   if (0!=(ret=RPI_do(&in,&out)))
      *npoints=out.l_param[0];
   return(ret);       
}
/*****************************************************************************/
/* move all points of an object relative to current position */

   RPIStub long
RPI_geo_move
   (RPI_HANDLE handle,RPI_OBJECT obj,float delta[3])
{             
   /* handle:       refgateway-handle
      obj:          object-handle
      delta:        dx,dy,dz local coords to move all points
      returns:      0 if failed
                    1 if ok
   */
   PLI_param in,out;
   long i;

   in.v_param[0]=handle;
   in.id=RPI_GEO_MOVE;
   in.v_param[1]=obj;
   for (i=0;i<3;i++)
     in.f_param[i]=delta[i];
   return(RPI_do(&in,&out));       
}
/*****************************************************************************/
/* set the position of an object */

   RPIStub long 
RPI_geo_set
   (RPI_HANDLE handle,RPI_OBJECT obj,float v[3])
{             
   /* handle:       refgateway-handle
      obj:          object-handle
      v:            new x,y, and z local coordinates for the object
      returns:      0 if failed
                    1 if ok
   */
   PLI_param in,out;
   long i;

   in.v_param[0]=handle;
   in.id=RPI_GEO_SET;
   in.v_param[1]=obj;
   for (i=0;i<3;i++)
     in.f_param[i]=v[i];
   return(RPI_do(&in,&out));       
}
/*****************************************************************************/
/* set the size factor of an object */

   RPIStub long
RPI_geo_size
   (RPI_HANDLE handle,RPI_OBJECT obj,float size)
{             
   /* handle:       refgateway-handle
      obj:          object-handle
      size:         new size factor for the object, default=1.0
      returns:      0 if failed
                    1 if ok
   */
   PLI_param in,out;

   in.v_param[0]=handle;
   in.id=RPI_GEO_SIZE;
   in.v_param[1]=obj;
   in.f_param[0]=size;
   return(RPI_do(&in,&out));       
}
/*****************************************************************************/
/* rotate an object to make it aim at a specified point */

   RPIStub long
RPI_geo_aim_to
   (RPI_HANDLE handle,RPI_OBJECT obj,float pt[3])
{             
   /* handle:       refgateway-handle
      obj:          object-handle
      pt:           point to aim at (world coordinates)
      returns:      0 if failed
                    1 if ok
   */
   PLI_param in,out;
   long i;

   in.v_param[0]=handle;
   in.id=RPI_GEO_AIM_TO;
   in.v_param[1]=obj;
   for (i=0;i<3;i++)
      in.f_param[i]=pt[i];
   return(RPI_do(&in,&out));       
}
/*****************************************************************************/
/* rotate an object local */

   RPIStub long
RPI_geo_rot
   (RPI_HANDLE handle,RPI_OBJECT obj,float angle,long axis)
{             
   /* handle:       refgateway-handle
      obj:          object-handle
      angle:        rotation angle (in degrees)
      axis:         local axis to rotate around (0=x,1=y,2=z)
      returns:      0 if failed
                    1 if ok
   */
   PLI_param in,out;

   in.v_param[0]=handle;
   in.id=RPI_GEO_ROT;
   in.v_param[1]=obj;
   if (axis<0)
      axis=0;
   else if (axis>2)
      axis=2;
   in.l_param[0]=axis;
   in.f_param[0]=angle;
   return(RPI_do(&in,&out));       
}
/*****************************************************************************/
/* rotate an object global */

   RPIStub long
RPI_geo_rot_global
   (RPI_HANDLE handle,RPI_OBJECT obj,float angle,long axis)
{             
   /* handle:       refgateway-handle
      obj:          object-handle
      angle:        rotation angle (in degrees)
      axis:         global axis to rotate around (0=x,1=y,2=z)
      returns:      0 if failed
                    1 if ok
   */
   PLI_param in,out;

   in.v_param[0]=handle;
   in.id=RPI_GEO_ROT_GLOBAL;
   in.v_param[1]=obj;
   if (axis<0)
      axis=0;
   else if (axis>2)
      axis=2;
   in.l_param[0]=axis;
   in.f_param[0]=angle;
   return(RPI_do(&in,&out));       
}

 /*****************************************************************************/

#if MONZOOM_VERSION >= 45

/*****************************************************************************/
/* set object orientation in degrees [-180..180] for z and y, -90 bis 90 for x */
/* rotate order is y,x,z axis */

   RPIStub long
RPI_geo_set_orientation
   (RPI_HANDLE handle,RPI_OBJECT obj,float angles[3])
{
   /* handle:       refgateway-handle
      obj:          object-handle
      angles:       orientation angles (in degrees)
      returns:      0 if failed
                    1 if ok
   */
   PLI_param in,out;

   in.v_param[0]=handle;
   in.id=RPI_GEO_SET_ORIENTATION;
   in.v_param[1]=obj;
   in.f_param[0]=angles[2];
   in.f_param[1]=angles[0];
   in.f_param[2]=angles[1];
   return(RPI_do(&in,&out));
}

/*****************************************************************************/
/* get object orientation in degrees*/

   RPIStub long
RPI_geo_get_orientation
   (RPI_HANDLE handle,RPI_OBJECT obj,float angles[3])
{
   /* handle:       refgateway-handle
      obj:          object-handle
      angles:       destination for orientation angles (in degrees)
      returns:      0 if failed
                    1 if ok
   */
   PLI_param in,out;
   long ret;

   in.v_param[0]=handle;
   in.id=RPI_GEO_GET_ORIENTATION;
   in.v_param[1]=obj;
   if (0!=(ret=RPI_do(&in,&out))) {
      angles[2]=out.f_param[0];
      angles[0]=out.f_param[1];
      angles[1]=out.f_param[2];
   }
   return(ret);
}
/*****************************************************************************/

#endif /* if MONZOOM_VERSION >= 45 */

/*****************************************************************************/
/* transform local object coordinates to global coordinates */

   RPIStub long 
RPI_geo_pt_to_world
   (RPI_HANDLE handle,RPI_OBJECT obj,float pt[3],float wp[3])
{             
   /* handle:       refgateway-handle
      obj:          object-handle
      pt:           local object coords to transform to world coords
      wp:           destination for world coordinates of point
      returns:      0 if failed
                    1 if ok
   */
   PLI_param in,out;
   long ret,i;

   in.v_param[0]=handle;
   in.id=RPI_GEO_PT_TO_WORLD;
   in.v_param[1]=obj;
   for (i=0;i<3;i++)
      in.f_param[i]=pt[i];
   if (0!=(ret=RPI_do(&in,&out))) {
      for (i=0;i<3;i++)
         wp[i]=out.f_param[i];
   }
   return(ret);
}
/*****************************************************************************/
/* transform global coordinates to local object coordinates */

   RPIStub long
RPI_geo_pt_from_world
   (RPI_HANDLE handle,RPI_OBJECT obj,float wp[3],float pt[3])
{             
   /* handle:       refgateway-handle
      obj:          object-handle
      wp:           world coordinates to transform to local coords
      pt:           destination for local object coords
      returns:      0 if failed
                    1 if ok
   */
   PLI_param in,out;
   long ret,i;

   in.v_param[0]=handle;
   in.id=RPI_GEO_PT_FROM_WORLD;
   in.v_param[1]=obj;
   for (i=0;i<3;i++)
      in.f_param[i]=wp[i];
   if (0!=(ret=RPI_do(&in,&out))) {
      for (i=0;i<3;i++)
         pt[i]=out.f_param[i];
   }
   return(ret);       
}
/*****************************************************************************/
/* get the position of an object relatively to its father (if any) */

   RPIStub long
RPI_geo_get_pos
   (RPI_HANDLE handle,RPI_OBJECT obj,float pt[3])
{             
   /* handle:       refgateway-handle
      obj:          object-handle
      pt:           destination for local object coords
      returns:      0 if failed
                    1 if ok
   */
   PLI_param in,out;
   long ret,i;

   in.v_param[0]=handle;
   in.id=RPI_GEO_GET_POS;
   in.v_param[1]=obj;
   if (0!=(ret=RPI_do(&in,&out))) {
      for (i=0;i<3;i++)
         pt[i]=out.f_param[i];
   }
   return(ret);       
}
/*****************************************************************************/
/* get the position of an object in world coordinates */

   RPIStub long
RPI_geo_get_wk_pos
   (RPI_HANDLE handle,RPI_OBJECT obj,float pt[3])
{             
   /* handle:       refgateway-handle
      obj:          object-handle
      pt:           destination for world coords
      returns:      0 if failed
                    1 if ok
   */
   PLI_param in,out;
   long ret,i;

   in.v_param[0]=handle;
   in.id=RPI_GEO_GET_WK_POS;
   in.v_param[1]=obj;
   if (0!=(ret=RPI_do(&in,&out))) {
      for (i=0;i<3;i++)
         pt[i]=out.f_param[i];
   }
   return(ret);       
}
/*****************************************************************************/
/* set the position of an object in world coordinates */

   RPIStub long 
RPI_geo_set_wk_pos
   (RPI_HANDLE handle,RPI_OBJECT obj,float v[3])
{             
   /* handle:       refgateway-handle
      obj:          object-handle
      v:            new x,y, and z world coordinates for the object
      returns:      0 if failed
                    1 if ok
   */
   PLI_param in,out;
   long i;

   in.v_param[0]=handle;
   in.id=RPI_GEO_SET_WK_POS;
   in.v_param[1]=obj;
   for (i=0;i<3;i++)
     in.f_param[i]=v[i];
   return(RPI_do(&in,&out));       
}
/*****************************************************************************/
/* get the local 3D bounding box of an object */

   RPIStub long 
RPI_geo_get_bbox
   (RPI_HANDLE handle,RPI_OBJECT obj,float min[3],float max[3])
{             
   /* handle:       refgateway-handle
      obj:          object-handle
      min,max:      destination for coordinates of bounding box in local
                    coordinates (xmin,ymin,zmin,xmax,ymax,zmax)
      returns:      0 if failed
                    1 if ok
   */
   PLI_param in,out;
   long ret,i;

   in.v_param[0]=handle;
   in.id=RPI_GEO_GET_BBOX;
   in.v_param[1]=obj;
   if (0!=(ret=RPI_do(&in,&out))) {
      for (i=0;i<3;i++) {
         min[i]=out.f_param[i];
         max[i]=out.f_param[i+3];
      }
   }
   return(ret);       
}
/*****************************************************************************/
/* get the global 3D bounding box of an object */

   RPIStub long 
RPI_geo_get_wk_bbox
   (RPI_HANDLE handle,RPI_OBJECT obj,float min[3],float max[3])
{             
   /* handle:       refgateway-handle
      obj:          object-handle
      min,max:      destination for coordinates of bounding box in local
                    coordinates (xmin,ymin,zmin,xmax,ymax,zmax)
      returns:      0 if failed
                    1 if ok
   */
   PLI_param in,out;
   long ret,i;

   in.v_param[0]=handle;
   in.id=RPI_GEO_GET_WK_BBOX;
   in.v_param[1]=obj;
   if (0!=(ret=RPI_do(&in,&out))) {
      for (i=0;i<3;i++) {
         min[i]=out.f_param[i];
         max[i]=out.f_param[i+3];
      }
   }
   return(ret);
}
/*****************************************************************************/
/* get the 2D bounding box of an object on a reflections display */

   RPIStub long
RPI_geo_get_2d_bbox
   (RPI_HANDLE handle,RPI_OBJECT obj,long display_id,long min[2],long max[2])
{             
   /* handle:       refgateway-handle
      obj:          object-handle
      display_id:   ID-number of an reflections display
      min,max:      destination for coordinates of bounding box in
                    display coordinates (xmin,ymin,xmax,ymax)
      returns:      0 if failed
                    1 if ok
   */
   PLI_param in,out;
   long ret,i;

   in.v_param[0]=handle;
   in.id=RPI_GEO_GET_2D_BBOX;
   in.v_param[1]=obj;
   in.l_param[0]=display_id;
   ret=RPI_do(&in,&out); 
   if (ret) {
      for (i=0;i<2;i++) {
         min[i]=out.l_param[i];
         max[i]=out.l_param[i+2];
      }
   }
   return(ret);       
}
/*****************************************************************************/
/* get the index number of an geo-object's point which is next to */
/* a specified coordinate on a reflections-display */

   RPIStub long
RPI_geo_get_next_pt_2d
   (RPI_HANDLE handle,RPI_OBJECT obj,long display_id,long x,long y,long *point_nr)

{             
   /* handle:       refgateway-handle
      obj:          object-handle
      display_id:   ID-number of an reflections display
      x,y:          display-coordinates to test points against
      pointnr:      destination for number of point which is next to x,y
      returns:      0 if failed
                    1 if ok
   */
   PLI_param in,out;
   long ret;

   in.v_param[0]=handle;
   in.id=RPI_GEO_NEXT_PT_2D;
   in.v_param[1]=obj;
   in.l_param[0]=display_id;
   in.l_param[1]=x;
   in.l_param[2]=y;
   if (0!=(ret=RPI_do(&in,&out)))
      *point_nr=out.l_param[0];
   return(ret);       
}
/*****************************************************************************/
/* get a list of an geo-objext's points which are inside a specified */
/* rectangle on a reflections-display */

   RPIStub long 
RPI_geo_get_pts_in_rect_2d
   (RPI_HANDLE handle,RPI_OBJECT obj,long display_id,
   long xmin,long ymin,long xmax,long ymax,
   long maxpts,long *pt_array,long *npts)
{             
   /* handle:       refgateway-handle
      obj:          object-handle
      display_id:   ID-number of an reflections display
      xmin,ymin:    left and upper coords of 2D rectangle
      xmax,ymax:    right and bottom coords of 2D rectangle
      maxpts:       the number of longs that can be stored in pt_array
      pt_array:     destination array for the point index numbers
      npts:         destination for the number of points inside the rectangle
      returns:      0 if failed
                    1 if ok
   */
   PLI_param in,out;
   long ret;

   in.v_param[0]=handle;
   in.id=RPI_GEO_PTS_IN_RECT_2D;
   in.v_param[1]=obj;
   in.l_param[0]=display_id;
   in.l_param[1]=xmin;
   in.l_param[2]=ymin;
   in.l_param[3]=xmax;
   in.l_param[4]=ymax;
   in.l_param[5]=maxpts;
   in.v_param[2]=(void *)pt_array;
   if (0!=(ret=RPI_do(&in,&out)))
      *npts=out.l_param[0];
   return(ret);       
}
/*****************************************************************************/
/* get the display coords of a point of a geo-object on a reflections-display */

   RPIStub long
RPI_geo_get_pt_2d_koord
   (RPI_HANDLE handle,RPI_OBJECT obj,long display_id,long point_nr,long *x,long *y)
{             
   /* handle:       refgateway-handle
      obj:          object-handle
      display_id:   ID-number of an reflections display
      pointnr:      index number of point
      x,y:          destination for the display coordinates of point
      returns:      0 if failed
                    1 if ok
   */
   PLI_param in,out;
   long ret;

   in.v_param[0]=handle;
   in.id=RPI_GEO_PT_2D_KOORD;
   in.v_param[1]=obj;
   in.l_param[0]=display_id;
   in.l_param[1]=point_nr;
   if (0!=(ret=RPI_do(&in,&out))) {
      *x=out.l_param[0];
      *y=out.l_param[1];
   }
   return(ret);       
}
/*****************************************************************************/
/* get the display coords of a point of a geo-object on a reflections-display */
/* this is an enhanced version of RPI_geo_get_pt_2d_koord, which works without */
/* a previous call to plot_pkl() */

   RPIStub long
RPI_geo_projekt_pt
   (RPI_HANDLE handle,RPI_OBJECT obj,long display_id,long point_nr,long *x,long *y)
{             
   /* handle:       refgateway-handle
      obj:          object-handle
      display_id:   ID-number of an reflections display
      pointnr:      index number of point
      x,y:          destination for the display coordinates of point
      returns:      0 if failed
                    1 if ok
   */
   PLI_param in,out;
   long ret;

   in.v_param[0]=handle;
   in.id=RPI_GEO_PROJEKT_PT;
   in.v_param[1]=obj;
   in.l_param[0]=display_id;
   in.l_param[1]=point_nr;
   if (0!=(ret=RPI_do(&in,&out))) {
      *x=out.l_param[0];
      *y=out.l_param[1];
   }
   return(ret);       
}
/*****************************************************************************/
/* plot a single point of an object in a reflections display */

   RPIStub long
RPI_geo_plot_pt
   (RPI_HANDLE handle,RPI_OBJECT obj,long display_id,long point_nr,long art)
{             
   /* handle:       refgateway-handle
      obj:          object-handle
      display_id:   ID-number of an reflections display
      pointnr:      index number of point
      art:          select state of point, 0=normal, 1=selected
      returns:      0 if failed
                    1 if ok
   */
   PLI_param in,out;

   in.v_param[0]=handle;
   in.id=RPI_GEO_PLOT_PT;
   in.v_param[1]=obj;
   in.l_param[0]=display_id;
   in.l_param[1]=point_nr;
   in.l_param[2]=art;
   return(RPI_do(&in,&out));       
}
/*****************************************************************************/
/* set the size of the points buffer of a geo-object */

   RPIStub long
RPI_geo_realloc_pts
   (RPI_HANDLE handle,RPI_OBJECT obj,long npts)
{
   /* handle:       refgateway-handle
      obj:          object-handle
      npts:         number of pts to reallocate
      returns:      0 if failed
                    1 if ok
   */
   PLI_param in,out;
 
   in.v_param[0]=handle;
   in.id=RPI_GEO_REALLOC_PTS;
   in.v_param[1]=obj;
   in.l_param[0]=npts;
   return(RPI_do(&in,&out));     
}
/*****************************************************************************/
/* request the number of sons of a geo-object */

   RPIStub long
RPI_geo_n_sons
   (RPI_HANDLE handle,RPI_OBJECT obj,long *nsons)
{
   /* handle:       refgateway-handle
      obj:          object-handle
      nsons:        destination for the number of sons
      returns:      0 if failed
                    1 if ok
   */
   PLI_param in,out;
   long ret;

   in.v_param[0]=handle;
   in.id=RPI_GEO_N_SONS;
   in.v_param[1]=obj;
   if (0!=(ret=RPI_do(&in,&out)))
      *nsons=out.l_param[0];
   return(ret);     
}
/*****************************************************************************/
/* request the father of a geo-object */

   RPIStub long
RPI_geo_get_father
   (RPI_HANDLE handle,RPI_OBJECT obj,RPI_OBJECT *father)
{
   /* handle:       refgateway-handle
      obj:          object-handle
      father:       destination for the father object-handle
      returns:      0 if failed
                    1 if ok
   */
   PLI_param in,out;
   long ret;

   in.v_param[0]=handle;
   in.id=RPI_GEO_GET_FATHER;
   in.v_param[1]=obj;
   if (0!=(ret=RPI_do(&in,&out)))
      *father=out.v_param[0];
   return(ret);     
}
/*****************************************************************************/
/* request a son of a geo-object */

   RPIStub long
RPI_geo_get_son
   (RPI_HANDLE handle,RPI_OBJECT obj,long son_nr,RPI_OBJECT *son)
{
   /* handle:       refgateway-handle
      obj:          object-handle
      son_nr:       the nr of the requested son 
      son:          destination for the son object-handle
      returns:      0 if failed
                    1 if ok
   */
   PLI_param in,out;
   long ret;

   in.v_param[0]=handle;
   in.id=RPI_GEO_GET_SON;
   in.v_param[1]=obj;
   in.l_param[0]=son_nr;
   if (0!=(ret=RPI_do(&in,&out)))
      *son=out.v_param[0];
   return(ret);     
}
/*****************************************************************************/
/* adds a son to a geo-object */

   RPIStub long
RPI_geo_insert_son
   (RPI_HANDLE handle,RPI_OBJECT obj,RPI_OBJECT son)
{
   /* handle:       refgateway-handle
      obj:          object-handle
      son:          son object-handle
      returns:      0 if failed
                    1 if ok
   */
   PLI_param in,out;

   in.v_param[0]=handle;
   in.id=RPI_GEO_INSERT_SON;
   in.v_param[1]=obj;
   in.v_param[2]=son;
   return(RPI_do(&in,&out));
}
/*****************************************************************************/
/* removes a geo-object from its hierarchy */

#define RPI_geo_delete_from_hierarchie RPI_geo_delete_from_hirarchie

   RPIStub long
RPI_geo_delete_from_hirarchie
   (RPI_HANDLE handle,RPI_OBJECT obj)
{
   /* handle:       refgateway-handle
      obj:          object-handle
      returns:      0 if failed
                    1 if ok
   */
   PLI_param in,out;

   in.v_param[0]=handle;
   in.id=RPI_GEO_DELETE_FROM_HIRARCHIE;
   in.v_param[1]=obj;
   return(RPI_do(&in,&out));
}
/*****************************************************************************/
