/*****************************************************************************/
/*          refgateway stub functions for proper and save usage              */
/*                  of the RPI_do(in,out) commands.                          */
/*****************************************************************************/

/* title:    kugel object functions */
/* author:   Carsten Fuchs */
/* version:  1.0, 30.1.98 */
/* updated:  23.3.98 by RG */
/* updated:  17.11.99 by RG */
/* updated:  2.3.2000 by RG */

/*****************************************************************************/
/* create a new sphere */

   RPIStub long
RPI_kug_new_kug
   (RPI_HANDLE handle,RPI_OBJECT obj,long ptnr,float rad,long *kugnr)
{
   /* handle:      refgateway-handle
      obj:         object-handle
      ptnr:        index number of the sphere midpoint
      rad:         radius of sphere
      kugnr:       destination for the index number of the new sphere
      returns:     0 if failed
                   1 if ok
   */
   PLI_param in,out;
   long ret;

   in.v_param[0]=handle;
   in.id=RPI_KUG_NEW_KUG;
   in.v_param[1]=obj;
   in.l_param[0]=ptnr;
   in.f_param[0]=rad;
   if (0!=(ret=RPI_do(&in,&out)))
      *kugnr=out.l_param[0];
   return(ret);
}
/*****************************************************************************/
/* delete an existing sphere */

   RPIStub long
RPI_kug_del_kug
   (RPI_HANDLE handle,RPI_OBJECT obj,long kugnr)
{
   /* handle:      refgateway-handle
      obj:         object-handle
      kugnr :      index number of the sphere to delete
      returns:     0 if failed
                   1 if ok
   */
   PLI_param in,out;

   in.v_param[0]=handle;
   in.id=RPI_KUG_DEL_KUG;
   in.v_param[1]=obj;
   in.l_param[0]=kugnr;
   return(RPI_do(&in,&out));
}
/*****************************************************************************/
/* query the number of spheres */

   RPIStub long
RPI_kug_nkugeln
   (RPI_HANDLE handle,RPI_OBJECT obj,long *nkug)
{
   /* handle:      refgateway-handle
      obj:         object-handle
      nkug :       destination for the number of spheres
      returns:     0 if failed
                   1 if ok
   */
   PLI_param in,out;
   long ret;

   in.v_param[0]=handle;
   in.id=RPI_KUG_NKUG;
   in.v_param[1]=obj;
   if (0!=(ret=RPI_do(&in,&out)))
      *nkug=out.l_param[0];
   return(ret);
}
/*****************************************************************************/
/* change the radius of a sphere */

   RPIStub long
RPI_kug_set_rad
   (RPI_HANDLE handle,RPI_OBJECT obj,long kugnr,float rad)
{
   /* handle:      refgateway-handle
      obj:         object-handle
      kugnr :      index number of the sphere 
      rad:         radius of sphere 
      returns:     0 if failed
                   1 if ok
   */
   PLI_param in,out;

   in.v_param[0]=handle;
   in.id=RPI_KUG_SET_RAD;
   in.v_param[1]=obj;
   in.l_param[0]=kugnr;
   in.f_param[0]=rad;
   return(RPI_do(&in,&out));
}
/*****************************************************************************/
/* query the radius of a sphere */

   RPIStub long
RPI_kug_get_rad
   (RPI_HANDLE handle,RPI_OBJECT obj,long kugnr,float *rad)
{
   /* handle:      refgateway-handle
      obj:         object-handle
      kugnr :      index number of the sphere 
      rad:         destination of radius of sphere 
      returns:     0 if failed
                   1 if ok
   */
   PLI_param in,out;
   long ret;

   in.v_param[0]=handle;
   in.id=RPI_KUG_GET_RAD;
   in.v_param[1]=obj;
   in.l_param[0]=kugnr;
   if (0!=(ret=RPI_do(&in,&out)))
     *rad=out.f_param[0];
   return(ret);
}
/*****************************************************************************/
/* query the pt_index of the midpoint of a sphere */

   RPIStub long
RPI_kug_get_pt_nr
   (RPI_HANDLE handle,RPI_OBJECT obj,long kugnr,long *pt_nr)
{
   /* handle:      refgateway-handle
      obj:         object-handle
      kugnr :      index number of the sphere 
      ptnr:        destination of pointnr of sphere 
      returns:     0 if failed
                   1 if ok
   */
   PLI_param in,out;
   long ret;

   in.v_param[0]=handle;
   in.id=RPI_KUG_GET_KUG_PKTNR;
   in.v_param[1]=obj;
   in.l_param[0]=kugnr;
   if (0!=(ret=RPI_do(&in,&out)))
     *pt_nr=out.l_param[0];
   return(ret);
}
/*****************************************************************************/
/* set the material-object of a number of spheres */

   RPIStub long
RPI_kug_set_kugeln_mat
   (RPI_HANDLE handle,RPI_OBJECT obj,long nkugeln,long *kug_nrn,RPI_OBJECT mat)
{
   /* handle:      refgateway-handle
      obj:         object-handle
      nkugeln:     number of spheres to assign material to
      kugnrn:      nkugeln indices of spheres to assign material to 
      mat:         material-object for spheres
      returns:     0 if failed
                   1 if ok
   */
   PLI_param in,out;

   in.v_param[0]=handle;
   in.id=RPI_KUG_SET_KUGELN_MAT;
   in.v_param[1]=obj;
   in.l_param[0]=nkugeln;
   in.v_param[2]=(void *)kug_nrn;
   in.v_param[3]=mat;
   return(RPI_do(&in,&out));
}
/*****************************************************************************/
/* set the material-object of a single sphere */

   RPIStub long
RPI_kug_set_kug_mat
   (RPI_HANDLE handle,RPI_OBJECT obj,long kugnr,RPI_OBJECT mat)
{
   /* handle:      refgateway-handle
      obj:         object-handle
      kugnr :      index number of the sphere
      mat:         material-object for sphere 
      returns:     0 if failed
                   1 if ok
   */
   PLI_param in,out;

   in.v_param[0]=handle;
   in.id=RPI_KUG_SET_KUG_MAT;
   in.v_param[1]=obj;
   in.l_param[0]=kugnr;
   in.v_param[2]=mat;
   return(RPI_do(&in,&out));
}
/*****************************************************************************/
/* get the material-object of a single sphere */

   RPIStub long
RPI_kug_get_kug_mat
   (RPI_HANDLE handle,RPI_OBJECT obj,long kugnr,RPI_OBJECT *mat)
{
   /* handle:      refgateway-handle
      obj:         object-handle
      kugnr:       index number of the sphere
      mat:         destination for material of sphere 
      returns:     0 if failed
                   1 if ok
   */
   PLI_param in,out;
   long ret;

   in.v_param[0]=handle;
   in.id=RPI_KUG_GET_KUG_MAT;
   in.v_param[1]=obj;
   in.l_param[0]=kugnr;
   if (0!=(ret=RPI_do(&in,&out)))
     *mat=out.v_param[0];
   return(ret);
}
/*****************************************************************************/
