/*****************************************************************************/
/*          refgateway stub functions for proper and save usage              */
/*                  of the RPI_do(in,out) commands.                          */
/*****************************************************************************/

/* title:    material-object functions */
/* author:   Carsten Fuchs */
/* version:  1.0, 16.9.97 */
/* updated:  25.3.98 by RG */
/* version:  1.2, 23.3.99 */
/* updated:  17.11.99 by RG */
/* updated:  25.2.2000 by RG */
/* updated:  2.3.2000 by RG */

/*****************************************************************************/
/* set the color of a material */

   RPIStub long
RPI_mat_set_col
   (RPI_HANDLE handle,RPI_OBJECT obj,float rgb[3])
{
   /* handle:       refgateway-handle
      obj:          material object-handle
      rgb:          new rgb color value (range 0.0..1.0)
      returns:      0 if failed
                    1 if ok
   */
   PLI_param in,out;
   long i;

   in.v_param[0]=handle;
   in.id=RPI_MAT_SET_COL;
   in.v_param[1]=obj;
   for (i=0;i<3;i++)
     in.f_param[i]=rgb[i];
   return(RPI_do(&in,&out));
}
/*****************************************************************************/
/* get the color of a material */

   RPIStub long
RPI_mat_get_col
   (RPI_HANDLE handle,RPI_OBJECT obj,float rgb[3])
{
   /* handle:       refgateway-handle
      obj:          material object-handle
      rgb:          destination of rgb color value (range 0.0..1.0)
      returns:      0 if failed
                    1 if ok
   */
   PLI_param in,out;
   long i,ret;

   in.v_param[0]=handle;
   in.id=RPI_MAT_GET_COL;
   in.v_param[1]=obj;
   if (0!=(ret=RPI_do(&in,&out))) {
     for (i=0;i<3;i++)
       rgb[i]=out.f_param[i];
   }
   return(ret);
}
/*****************************************************************************/
/* set the values of a reflections material */

   RPIStub long
RPI_surf_set_params
   (RPI_HANDLE handle,RPI_OBJECT obj,float param[5])
{
   /* handle:       refgateway-handle
      obj:          material object-handle
      param:        new material values
      returns:      0 if failed
                    1 if ok
   */
   PLI_param in,out;
   long i;

   in.v_param[0]=handle;
   in.id=RPI_SURF_SET_PARAMS;
   in.v_param[1]=obj;
   for (i=0;i<5;i++)
     in.f_param[i]=param[i];
   return(RPI_do(&in,&out));
}
/*****************************************************************************/
/* get the values of a reflections material */

   RPIStub long
RPI_surf_get_params
   (RPI_HANDLE handle,RPI_OBJECT obj,float param[5])
{
   /* handle:       refgateway-handle
      obj:          material object-handle
      param:        destination for the material values          
      returns:      0 if failed
                    1 if ok
   */
   PLI_param in,out;
   long i,ret;

   in.v_param[0]=handle;
   in.id=RPI_SURF_GET_PARAMS;
   in.v_param[1]=obj;
  
   if (0!=(ret=RPI_do(&in,&out))) {
      for (i=0;i<5;i++)
         param[i]=out.f_param[i];
   }
   return(ret);
}
/*****************************************************************************/
/* set the values of a reflections glanzlicht (highlight) */

   RPIStub long
RPI_surf_set_glanz
   (RPI_HANDLE handle,RPI_OBJECT obj,float glanzlicht[13])
{
   /* handle:       refgateway-handle
      obj:          material object-handle
      glanzlicht:   new highlight envelope (range 0.0 .. 1.0)
      returns:      0 if failed
                    1 if ok
   */
   PLI_param in,out;

   in.v_param[0]=handle;
   in.id=RPI_SURF_SET_GLANZ;
   in.v_param[1]=obj;
   in.v_param[2]=glanzlicht;
   return(RPI_do(&in,&out));
}
/*****************************************************************************/
/* get the values of a reflections glanzlicht (highlight) */

   RPIStub long
RPI_surf_get_glanz
   (RPI_HANDLE handle,RPI_OBJECT obj,float glanzlicht[13])
{
   /* handle:       refgateway-handle
      obj:          material object-handle
      glanzlicht:   destination for highlight envelope (range 0.0 .. 1.0)
      returns:      0 if failed
                    1 if ok
   */
   PLI_param in,out;

   in.v_param[0]=handle;
   in.id=RPI_SURF_GET_GLANZ;
   in.v_param[1]=obj;
   in.v_param[2]=glanzlicht;
   return(RPI_do(&in,&out));
}
/*****************************************************************************/

#if MONZOOM_VERSION >= 46

/*****************************************************************************/
/* get size of material-picture */

   RPIStub long
RPI_mat_get_pic_size
   (RPI_HANDLE handle,RPI_OBJECT obj,long *dx,long *dy)
{
   /* handle:       refgateway-handle
      obj:          material object-handle
      dx,dy:        destination of picture-size
      returns:      0 if failed
                    1 if ok
   */
   PLI_param in,out;
   long ret;

   in.v_param[0]=handle;
   in.id=RPI_MAT_GET_PIC_SIZE;
   in.v_param[1]=obj;
   if (0!=(ret=RPI_do(&in,&out))) {
      *dx=out.l_param[0];
      *dy=out.l_param[1];
   }
   return(ret);
}
/*****************************************************************************/
/* get a pointer to the DIB of a material picture */
   
   RPIStub long
RPI_mat_get_pic_data
   (RPI_HANDLE handle,RPI_OBJECT obj,unsigned char **rgb)
{
   /* handle:       refgateway-handle
      obj:          material object-handle
      rgb:          destination of pointer to rgb-data of material
                    rgb-data is in DIB format
      returns:      0 if failed
                    1 if ok
   */
   PLI_param in,out;

   in.v_param[0]=handle;
   in.id=RPI_MAT_GET_PIC_DATA;
   in.v_param[1]=obj;
   in.v_param[2]=rgb;
   return(RPI_do(&in,&out));
}
/*****************************************************************************/

#endif /* if MONZOOM_VERSION >= 46 */

/*****************************************************************************/
