/*****************************************************************************/
/*          refgateway stub functions for proper and save usage             */
/*                  of the RPI_do(in,out) commands.                         */
/*****************************************************************************/

/* title:    memory functions */
/* author:   Carsten Fuchs  */
/* version:  1.0, 16.9.97   */
/* updated:  23.3.98 by RG */
/* updated:  19.11.99 by RG */
/* updated:  2.3.2000 by RG */

/*****************************************************************************/
/* allocate memory */

   RPIStub void *
RPI_alloc_mem
   (RPI_HANDLE handle,long nbytes)
{
   /* handle:      refgateway-handle
      nbytes:      nr of bytes to allocate
      returns:     NULL if no memory available
                   ptr to allocated memory if ok
   */
   PLI_param in,out;
   void *ret=NULL;

   in.v_param[0]=handle;
   in.id=RPI_ALLOC_MEM;
   in.l_param[0]=nbytes;
   if (RPI_do(&in,&out))
     ret=out.v_param[0];
   return(ret);
}
/*****************************************************************************/
/* allocate and clear memory */

   RPIStub void *
RPI_alloc_clear
   (RPI_HANDLE handle,long nbytes)
{
   /* handle:      refgateway-handle
      nbytes:      nr of bytes to allocate
      returns:     NULL if no memory available
                   ptr to allocated and cleared memory if ok
   */
   PLI_param in,out;
   void *ret=NULL;

   in.v_param[0]=handle;
   in.id=RPI_ALLOC_MEM;
   in.l_param[0]=nbytes;
   if (RPI_do(&in,&out)) {
      if (0!=(ret=out.v_param[0])) memset(ret,0,nbytes);
   }
   return(ret);
}
/**********************************************************************************/
/* free allocated memory */

   RPIStub long 
RPI_free_mem
   (RPI_HANDLE handle,void *mem_pointer)
{
   /* handle:      refgateway-handle
      mem_points:  ptr to memory to be freed
      returns:     0 if failed
                   1 if ok
   */
   PLI_param in,out;

   in.v_param[0]=handle;
   in.id=RPI_FREE_MEM;
   in.v_param[1]=mem_pointer;
   return(RPI_do(&in,&out));
}
/**********************************************************************************/
/* allocate a heap */

   RPIStub RPI_HEAP
RPI_new_heap
   (RPI_HANDLE handle)
{
   /* handle:      refgateway-handle
      returns:     NULL if no heap can be created
                   ptr to heap, if ok
   */
   PLI_param in,out;
   void *ret=NULL;

   in.v_param[0]=handle;
   in.id=RPI_NEW_HEAP;
   if (RPI_do(&in,&out))
     ret=out.v_param[0];
   return(ret);
}
/*****************************************************************************/
/* close a heap (free all memory of heap) */

   RPIStub long 
RPI_close_heap
   (RPI_HANDLE handle,RPI_HEAP heap)
{
   /* handle:      refgateway-handle
      heap:        ptr to a heap
      returns:     0 if heap is illegal
                   1 if ok
   */
   PLI_param in,out;
   
   in.v_param[0]=handle;
   in.v_param[1]=heap;
   in.id=RPI_CLOSE_HEAP;
   return(RPI_do(&in,&out));
}
/*****************************************************************************/
/* allocate heap-memory */

   RPIStub void *
RPI_heap_alloc_mem
   (RPI_HANDLE handle,RPI_HEAP heap,long nbytes)
{
   /* handle:      refgateway-handle
      heap:        ptr to a heap
      nbytes:      nr of bytes to allocate
      returns:     NULL if no memory available or if heap is illegal
                   ptr to allocated memory if ok
   */
   PLI_param in,out;
   void *ret=NULL;

   in.v_param[0]=handle;
   in.id=RPI_HEAP_ALLOC_MEM;
   in.v_param[1]=heap;
   in.l_param[0]=nbytes;
   if (RPI_do(&in,&out))
     ret=out.v_param[0];
   return(ret);
}
/*****************************************************************************/
/* allocate and clear heap-memory */

   RPIStub void *
RPI_heap_alloc_clear
   (RPI_HANDLE handle,RPI_HEAP heap,long nbytes)
{
   /* handle:      refgateway-handle
      heap:        ptr to a heap
      nbytes:      nr of bytes to allocate
      returns:     NULL if no memory available or if heap is illegal
                   ptr to allocated memory if ok
   */
   PLI_param in,out;
   void *ret=NULL;

   in.v_param[0]=handle;
   in.id=RPI_HEAP_ALLOC_MEM;
   in.v_param[1]=heap;
   in.l_param[0]=nbytes;
   if (RPI_do(&in,&out)) {
     if (0!=(ret=out.v_param[0])) memset(ret,0,nbytes);
   }
   return(ret);
}
/*****************************************************************************/
/* free allocated heap-memory */

   RPIStub long 
RPI_heap_free_mem
   (RPI_HANDLE handle,RPI_HEAP heap,void *mem_pointer)
{
   /* handle:      refgateway-handle
      heap:        ptr to a heap
      mem_pointer: ptr to memory to be freed
      returns:     0 if failed (illegal heap)
                   1 if ok
   */
   PLI_param in,out;

   in.v_param[0]=handle;
   in.id=RPI_HEAP_FREE_MEM;
   in.v_param[1]=heap;
   in.v_param[2]=mem_pointer;
   return(RPI_do(&in,&out));
}
/**********************************************************************************/
