/*****************************************************************************/
/*                        monzoom plugin skeleton                            */
/*****************************************************************************/

/* title:    afx plugin skeleton */
/* author:   Reinhard Grams  */
/* version:  1.0, 24.2.98 */
/* updated:  5.6.98 */
/* updated:  28.2.2000 */
/* updated:  3.3.2000 by RG */

/*****************************************************************************/
/* this is what you have to do to customize this skeleton: */
/* 1. change the name of the plugin as defined below */
/* 2. place your general init and cleanup code in */
/* RefPlugConnect() and RefPlugCloseResources() */
/* 3. edit the requester and variable definitions in AfxInitVariables */
/* 4. edit the AfxData struct to hold the needed variables */
/* 5. place your animation init and cleanup code in Start() and Finish(), */
/* after removing the example source in that functions */
/* 6. place the animation code in PreRender() and the cleanup code for */
/* each frame in AfterRender() */
/* 7. depending on the number of temp object, edit PLI_AFX_GET_OBJS and */
/* PLI_AFX_SET_OBJS to get them stored and reloaded when the user cancels */
/* and later resumes the animation */
/* 8. test your piece of work and please send a copy to Oberland */
/*****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

/*****************************************************************************/
/* error codes and messages, add your own errors at the specified places */

#define PLUG_OK                  0

#define PLUG_INTERNAL_ERR        0
#define PLUG_NO_MEM_ERR          1
#define PLUG_OPEN_FILE_ERR       2
#define PLUG_READ_ERR            3
#define PLUG_WRITE_ERR           4
#define PLUG_FILE_ERR            5
#define PLUG_CONFIG_ERR          6
#define PLUG_CONFIG_OPEN_ERR     7
#define PLUG_OPEN_REQ_ERR        8
#define PLUG_GET_VALUE_ERR       9
#define PLUG_REF_DIR_ERR         10
#define PLUG_ALLOC_VARS_ERR      11
#define PLUG_DEFINE_VAR_ERR      12
#define PLUG_SET_VALUE_ERR       13
#define PLUG_USER_BREAK_ERR      14
#define PLUG_GET_NAME_ERR        15
#define PLUG_NO_OBJECT_ERR       16
#define PLUG_GET_OBJECTS_ERR     17
#define PLUG_CREATE_OBJECT_ERR   18
#define PLUG_NO_POINTS_ERR       19
#define PLUG_SET_POINT_ERR       20
#define PLUG_SET_NAME_ERR        21
#define PLUG_SET_LOCK_ERR        22
#define PLUG_SET_DATA_ERR        23
#define PLUG_GET_DATA_ERR        24
#define PLUG_NO_DATA_ERR         25
#define PLUG_DLL_VERSION_ERR     26
/* add your error codes here */

#define ERROR_MESSAGES 27
/* increase this constant if you added error codes */

char  *mess_text[ERROR_MESSAGES]=
{
   "Interner Fehler",
   "Kein Speicher mehr frei",
   "Kann File nicht ffnen",
   "Kann Datei nicht lesen",
   "Kann Datei nicht schreiben",
   "Fehler beim Dateizugriff",
   "Fehlerhafte Konfigurationsdatei",
   "Kann Konfigurationsdatei nicht ffnen",
   "Kann Requester nicht ffnen",
   "Kann Werte nicht abfragen",
   "Kann Verzeichnis nicht ermitteln",
   "Kann Variablen nicht anlegen",
   "Kann Variablentyp nicht definieren",
   "Kann Wert nicht zuweisen",
   "Abbruch durch Anwender",
   "Kann Name nicht abfragen",
   "Kein Objekt zur Auswahl",
   "Kann Objekte nicht abfragen",
   "Kann Objekt nicht erzeugen",
   "Objekt hat keine Punkte",
   "Kann Punkt nicht verschieben",
   "Kann Namen nicht setzen",
   "Kann Lock fr Objekt nicht setzen",
   "Kann Userdaten nicht eintragen",
   "Kann Userdaten nicht abfragen",
   "Keine Userdaten vorhanden",
   /* add your error message here */
};

/*****************************************************************************/

void *HND;

/*****************************************************************************/
/* set this to 43 or less if you compile for an older version */

#define MONZOOM_VERSION    45
#define MONZOOM_REVISION   0

#define DLL_VERSION ((MONZOOM_VERSION<<16) | MONZOOM_REVISION)

/*****************************************************************************/
/* define this if you use the skeleton's standard config io */

#undef PLUGIN_STANDARD_CONFIG_HANDLING

/*****************************************************************************/
/* undef this if you don't want to use the MonStubs.dll */

#define PLUGIN_STUBS_LIBRARY

/*****************************************************************************/
/* use 1 to get the stubs included, 0 otherwise */
/* this defs are used in "rgw_all.h" */

#define INCLUDE_GUI_STUBS           1
#define INCLUDE_PKL_STUBS           0
#define INCLUDE_GEO_EDIT_STUBS      0
#define INCLUDE_OPERATIONS_STUBS    0
#define INCLUDE_CREATE_STUBS        0
#define INCLUDE_OBJECT_STUBS        1
#define INCLUDE_GEO_STUBS           0
#define INCLUDE_EDGES_STUBS         0
#define INCLUDE_TRIANGLE_STUBS      0
#define INCLUDE_MATERIALS_STUBS     0
#define INCLUDE_MEMORY_STUBS        1
#define INCLUDE_FILE_STUBS          0
#define INCLUDE_POLYGON_STUBS       0
#define INCLUDE_PARTICLE_STUBS      0
#define INCLUDE_SPHERES_STUBS       0
#define INCLUDE_DATA_STUBS          0
#define INCLUDE_DISPLAY_STUBS       0
#define INCLUDE_AFTERBURNER_STUBS   0
#define INCLUDE_AFX_STUBS           1
#define INCLUDE_CAMERA_STUBS        0

/*****************************************************************************/
/* this includes all files needed */

#include <rgw_all.h>

/*****************************************************************************/
/* the name and version of this plugin */

#define PLUG_NAME             "AnimSkel"
#define PLUG_VERSION          1
#define PLUG_REVISION         0

/*****************************************************************************/

#define REQ1_TITLE "Objekt 1"
#define REQ2_TITLE "Objekt 2"

#define PAR1_NAME  "Param 1"
#define PAR2_NAME  "Param 2"

/*****************************************************************************/

#define AFX_INT_VARS    1
#define AFX_FLOAT_VARS  2
#define AFX_OBJECT_VARS 2

typedef struct {

   void      *heap;
   float     dummy;

} AfxData;

#define PLUG_OBJECT_ID PARTIKEL_OBJEKT_ID

/*****************************************************************************/
/* get the error message of error code */

   void
GetErrorMessage
   (long err,char *buffer)
{
   if (err>0 && err<ERROR_MESSAGES) 
      sprintf(buffer,"%s: %s",PLUG_NAME,mess_text[err]);
   else 
      sprintf(buffer,"%s: %s %ld",PLUG_NAME,mess_text[PLUG_INTERNAL_ERR],err);
}

/*****************************************************************************/
/* this function is called at first when PLI_CONNECT arrives */

   long 
RefPlugConnect
   (void)
{
   long err=PLUG_OK;

   /* place your setup code here */

   return(err);
}

/*****************************************************************************/
/* this function is called when PLI_CLOSE_RESOURCES arrives */

   long
RefPlugCloseResources
   (void)
{
   long err=PLUG_OK;

   /* place your general cleanup code here */

   return(err);
}

/*****************************************************************************/
/* create & remove an indexed array for the requester */

typedef struct {

   void *data;
   void **index;
   long size,entries;

} Array;

   static Array *
RemoveHeapIndexedArray
   (Array *array,void *heap)
{
   if (array) {
      if (array->data) RPI_heap_free_mem(HND,heap,array->data);
      if (array->index) RPI_heap_free_mem(HND,heap,array->index);
      RPI_heap_free_mem(HND,heap,array);
   }
   return(NULL);
}

   static Array *
CreateHeapIndexedArray
   (long size,long entries,void *heap)
{
   Array *array=NULL;
   long e,pos; char *d,**i;

   if (size>0 && entries>0) {

      if (!(array=RPI_heap_alloc_clear(HND,heap,sizeof(Array)))) {
         goto ErrorExit;
      }
      if (!(array->data=RPI_heap_alloc_mem(HND,heap,entries*size))) {
         array=RemoveHeapIndexedArray(array,heap);
         goto ErrorExit;
      }
      if (!(array->index=RPI_heap_alloc_mem(HND,heap,entries*sizeof(void *)))) {
         array=RemoveHeapIndexedArray(array,heap);
         goto ErrorExit;
      }
      for (d=(char *)array->data,i=(char **)array->index,pos=0,e=0;e<entries;e++) {
         i[e]=&d[pos]; pos+=size;
      }
   }

ErrorExit:
   return(array);
}

/*****************************************************************************/
/* let the user select an object of a given id */

   long
DoObjectsRequester
   (char *title,long id,long subclass,RPI_OBJECT *object)
{
   long err=PLUG_OK;
   long e,all,entries,selected,obj_id;
   RPI_OBJECT *objects=NULL,*avail=NULL;
   Array *array=NULL;
   void *heap=NULL;

   *object=NULL;

   if (!(heap=RPI_new_heap(HND))) {
      err=PLUG_NO_MEM_ERR; goto ErrorExit;
   }
   if (!(all=RPI_n_objs(HND,id,subclass))) {
      err=PLUG_NO_OBJECT_ERR; goto ErrorExit;
   }
   if (all<1) {
      err=PLUG_NO_OBJECT_ERR; goto ErrorExit;
   }
   if (!(objects=RPI_heap_alloc_clear(HND,heap,all*sizeof(RPI_OBJECT)))) {
      err=PLUG_NO_MEM_ERR; goto ErrorExit;
   }
   if (!(avail=RPI_heap_alloc_clear(HND,heap,all*sizeof(RPI_OBJECT)))) {
      err=PLUG_NO_MEM_ERR; goto ErrorExit;
   }
   if (!RPI_get_objs(HND,id,subclass,all,avail)) {
      err=PLUG_GET_VALUE_ERR; goto ErrorExit;
   }
   for (entries=0,e=0;e<all;e++) {
      if ((obj_id=RPI_obj_get_id(HND,avail[e]))==0) {
         err=PLUG_GET_VALUE_ERR; goto ErrorExit;
      }
      if (obj_id==id) { 
         objects[entries]=avail[e];
         entries++;
      }  
   }
   if (entries<1) {
      err=PLUG_NO_OBJECT_ERR; goto ErrorExit;
   }
   if (!(array=CreateHeapIndexedArray(REFLECTIONS_NAME_BUFFER_SIZE,entries,heap))) {
      err=PLUG_NO_MEM_ERR; goto ErrorExit;
   }
   for (e=0;e<entries;e++) {
      if (!RPI_obj_get_name(HND,objects[e],array->index[e])) {
         err=PLUG_GET_NAME_ERR; goto ErrorExit;
      }
   }
   if (!RPI_get_selection(HND,title,entries,(char **)array->index,&selected)) {
      err=PLUG_OPEN_REQ_ERR; goto ErrorExit;
   }
   if (selected>=0) {
      *object=objects[selected];
   }

ErrorExit:
   if (heap) RPI_close_heap(HND,heap);
   return(err);
}

/*****************************************************************************/
/* create the user data for the animFX */

   long
CreateAfxData
   (RPI_OBJECT afx,AfxData **result)
{
   long err=PLUG_OK;
   void *heap;
   AfxData *data;

   *result=NULL;

   if (!(heap=RPI_new_heap(HND))) {
      err=PLUG_NO_MEM_ERR; goto ErrorExit;
   }
   if (!(data=RPI_heap_alloc_clear(HND,heap,sizeof(AfxData)))) {
      err=PLUG_NO_MEM_ERR; goto ErrorExit;
   }
   if (!RPI_afx_set_user_data(HND,afx,(void *)data)) {
      err=PLUG_SET_DATA_ERR; goto ErrorExit;
   }
   data->heap=heap;
   *result=data;

ErrorExit:
   if (err && heap) RPI_close_heap(HND,heap);
   return(err);
}

/*****************************************************************************/
/* remove the user data for the animFX */

   void
RemoveAfxData
   (RPI_OBJECT afx)
{
   AfxData *data=NULL;

   if (RPI_afx_get_user_data(HND,afx,(void **)&data)) {
      if (data) {
         RPI_afx_set_user_data(HND,afx,NULL);
         RPI_close_heap(HND,data->heap);
      }
   }
}

/*****************************************************************************/
/* setup the afx object variables */

   long
AfxInitVariables
   (RPI_OBJECT afx)
{
   long err=PLUG_OK;
   long nr;

   if (!RPI_afx_alloc_intern_vars(HND,afx,AFX_INT_VARS,AFX_FLOAT_VARS,AFX_OBJECT_VARS)) {
      err=PLUG_ALLOC_VARS_ERR; goto ErrorExit;
   }
   /* zero all internal variables (just to be shure) */
   nr=0;
   while (nr<AFX_INT_VARS) {
      if (!RPI_afx_set_int(HND,afx,nr,0)) {
         err=PLUG_SET_VALUE_ERR; goto ErrorExit;
      }
      nr++;
   }
   nr=0;
   while (nr<AFX_FLOAT_VARS) {
      if (!RPI_afx_set_real(HND,afx,nr,0.0F)) {
         err=PLUG_SET_VALUE_ERR; goto ErrorExit;
      }
      nr++;
   }
   nr=0;
   while (nr<AFX_OBJECT_VARS) {
      if (!RPI_afx_set_obj(HND,afx,nr,NULL)) {
         err=PLUG_SET_VALUE_ERR; goto ErrorExit;
      }
      nr++;
   }
   /* setup the object variables */
   if (!RPI_afx_set_n_obj_vars(HND,afx,5)) { /* 2 + 3 for rgb */ 
      err=PLUG_ALLOC_VARS_ERR; goto ErrorExit;
   }
   if (!RPI_afx_define_obj_var(HND,afx,0,1,PAR1_NAME,0.0F,1.0F)) {
     err=PLUG_DEFINE_VAR_ERR; goto ErrorExit;
   }
   if (!RPI_afx_define_obj_var(HND,afx,1,1,PAR2_NAME,0.0F,1.0F)) {
     err=PLUG_DEFINE_VAR_ERR; goto ErrorExit;
   }
   /* an rgb entry uses 3 float vars! */
   if (!RPI_afx_define_obj_rgb_var(HND,afx,2)) {
     err=PLUG_DEFINE_VAR_ERR; goto ErrorExit;
   }

ErrorExit:
   return(err);
}

/*****************************************************************************/
/* setup the afx object and its variables */

   long
AfxInit
   (RPI_OBJECT afx)
{
   long err=PLUG_OK;
   RPI_OBJECT object=NULL;
   char name[128],name1[128],name2[128];

   #ifdef PLUGIN_STUBS_LIBRARY   
   unsigned long dll_version;
   dll_version=MonPlugsGetVersion();
   if (dll_version<DLL_VERSION) {
      err=PLUG_DLL_VERSION_ERR; goto ErrorExit;
   }
   #endif

   /* let the user select one object */
   if (err=DoObjectsRequester(REQ1_TITLE,PLUG_OBJECT_ID,INCLUDE_SUBCLASSES,&object)) {
      goto ErrorExit;
   }
   if (object==NULL) {
      err=PLUG_USER_BREAK_ERR; goto ErrorExit;
   }
   /* init the afx variables */
   if (err=AfxInitVariables(afx)) {
      goto ErrorExit;
   }
   /* set the internal variables */
   if (!RPI_afx_set_int(HND,afx,0,0)) { /* flag for resuming */
      err=PLUG_SET_VALUE_ERR; goto ErrorExit;
   }
   if (!RPI_afx_set_obj(HND,afx,0,object)) { /* the user selected object */
      err=PLUG_SET_VALUE_ERR; goto ErrorExit;
   }
   if (!RPI_afx_set_obj(HND,afx,1,NULL)) { /* the animfx object */
      err=PLUG_SET_VALUE_ERR; goto ErrorExit;
   }
   /* initialize object variables */
   if (!RPI_obj_set_var(HND,afx,0,0.0F)) { /* parm1 */
      err=PLUG_SET_VALUE_ERR; goto ErrorExit;
   }
   if (!RPI_obj_set_var(HND,afx,1,0.0F)) { /* parm2 */
      err=PLUG_SET_VALUE_ERR; goto ErrorExit;
   }
   /* rename afx object */
   if (!RPI_obj_get_name(HND,afx,name1)) {
      err=PLUG_GET_NAME_ERR; goto ErrorExit;
   }
   if (!RPI_obj_get_name(HND,object,name2)) {
      err=PLUG_GET_NAME_ERR; goto ErrorExit;
   }
   sprintf(name,"%s_%s",name1,name2);
   if (!RPI_obj_set_name(HND,afx,name)) {
      err=PLUG_SET_NAME_ERR; goto ErrorExit;
   }

ErrorExit:
   return(err);
}

/*****************************************************************************/
/* this function is called before the animation is rendered */

   long
Start
   (RPI_OBJECT afx,float t1,float t2,float t,long picnr,long npics,long wire)
{
   long err=PLUG_OK;
   RPI_OBJECT source,object;

   if (!RPI_afx_get_obj(HND,afx,0,&source)) {
      err=PLUG_GET_VALUE_ERR; goto ErrorExit;
   }
   if (!RPI_afx_get_obj(HND,afx,1,&object)) {
      err=PLUG_GET_VALUE_ERR; goto ErrorExit;
   }
   if (!object) {
      /* create the temp object */
      if (!(object=RPI_obj_new(HND,PARTIKEL_OBJEKT_ID,"AnimObj"))) {
         err=PLUG_CREATE_OBJECT_ERR; goto ErrorExit;
      }
      /* store the object */
      if (!RPI_afx_set_obj(HND,afx,1,object)) {
         err=PLUG_SET_VALUE_ERR; goto ErrorExit;
      }
      /* clear the resume flag */
      if (!RPI_afx_set_int(HND,afx,0,0)) {
         err=PLUG_SET_VALUE_ERR; goto ErrorExit;
      }
   }

ErrorExit:
   return(err);
}

/*****************************************************************************/
/* this function is called after rendering the animation */
/* time to clean up and remove temp objects */

   long
Finish
   (RPI_OBJECT afx,float t1,float t2,float t,long picnr,long npics,long wire)
{
   long err=PLUG_OK;
   RPI_OBJECT object,source;

   if (!RPI_afx_get_obj(HND,afx,0,&source)) {
      err=PLUG_GET_VALUE_ERR; goto ErrorExit;
   }
   if (!RPI_afx_get_obj(HND,afx,1,&object)) {
      err=PLUG_GET_VALUE_ERR; goto ErrorExit;
   }
   if (object) {
      /* clear the object */
      if (!RPI_afx_set_obj(HND,afx,1,NULL)) {
         err=PLUG_GET_VALUE_ERR; goto ErrorExit;
      }
      /* remove the object */
      if (!RPI_obj_free(HND,object)) {
         err=PLUG_GET_VALUE_ERR; goto ErrorExit;
      }
   }

ErrorExit:
   return(err);
}

/*****************************************************************************/
/* this function is called before rendering a frame */

   long
PreRender
   (RPI_OBJECT afx,float t1,float t2,float t,long picnr,long npics,long wire)
{
   long err=PLUG_OK;
   RPI_OBJECT source,object;
   long resume;
   float v1,v2,rgb[3];

   /* test for illegal conditions */
   if (npics<1) goto ErrorExit;
   if (t2<=t1) goto ErrorExit;
   if (t<t1 || t>t2) goto ErrorExit;

   /* test if frame is allready generated */
   if (!RPI_afx_get_int(HND,afx,0,&resume)) {
      err=PLUG_GET_VALUE_ERR; goto ErrorExit;
   }
   if (resume) { /* frame is allready generated */
      if (!RPI_afx_set_int(HND,afx,0,0)) {
         err=PLUG_SET_VALUE_ERR; goto ErrorExit;
      }
      return(PLUG_OK);
   }
   /* get the user selected object */
   if (!RPI_afx_get_obj(HND,afx,0,&source)) {
      err=PLUG_GET_VALUE_ERR; goto ErrorExit;
   }
   /* get the generated object */
   if (!RPI_afx_get_obj(HND,afx,1,&object)) {
      err=PLUG_GET_VALUE_ERR; goto ErrorExit;
   }
   /* get the interpolated variables */
   if (!RPI_obj_get_var(HND,afx,0,&v1)) {
      err=PLUG_GET_VALUE_ERR; goto ErrorExit;
   }
   if (!RPI_obj_get_var(HND,afx,1,&v2)) {
      err=PLUG_GET_VALUE_ERR; goto ErrorExit;
   }
   /* get the rgb values */
   if (!RPI_obj_get_var(HND,afx,2,&rgb[0])) {
      err=PLUG_GET_VALUE_ERR; goto ErrorExit;
   }
   if (!RPI_obj_get_var(HND,afx,3,&rgb[1])) {
      err=PLUG_GET_VALUE_ERR; goto ErrorExit;
   }
   if (!RPI_obj_get_var(HND,afx,4,&rgb[2])) {
      err=PLUG_GET_VALUE_ERR; goto ErrorExit;
   }

   /* place your animFX code here */

ErrorExit:
   return(err);
}

/*****************************************************************************/
/* this function is called when the rendering of a frame is done */

   long
AfterRender
   (RPI_OBJECT afx,float t1,float t2,float t,long picnr,long npics,long wire)
{
   long err=PLUG_OK;

   /* perform cleanup here (if necessary) */

   return(err);   
}

/*****************************************************************************/

   long REFPROG
RPI_message
   (PTR0 PLI_param *in,PTR1 PLI_param *out)
{
   long err=PLUG_OK;
   char mess[128];

   #ifdef AMIGA
   if (!libs_installed) return(RPI_MESSAGE_UNKNOWN);
   #endif

   switch(in->id) {
         
      case FPLUG_INIT_MESSAGE:
         /* reflection has loaded us and passes the handle to us */
         HND=in->v_param[0];  
      break;

      case PLI_TOUCH:
         /* currently not used for afx plugins */
      break;

      case PLI_CONNECT:
         /* open amiga libraries */
         #include <AmiOpen.c>
         /* the plugin has been selected */
         if (err=RefPlugConnect()) {
            goto ErrorExit;
         }
      break;

      case PLI_CLOSE_RESOURCES:
         /* the plugin is going to be removed */
         err=RefPlugCloseResources();
      break;

      case FPLUG_CLOSE_MESSAGE:
         /* this is the end, my friend.. the shop is closed now! */
         HND=NULL;
          /* close the amiga libraries */
         #include <AmiClose.c>
     break;

      case PLI_AFX: 
         /* are we really an afx plugin? YES! */
         strcpy(out->s_param[0],"ANIM_FX");
      break;

      case PLI_AFX_INIT:
         /* initialize the objects and variables */
         if (err=AfxInit(in->v_param[0])) {
            goto ErrorExit;
         }
      break;

      case PLI_AFX_INIT_READ:
         if (err=AfxInitVariables(in->v_param[0])) {
            goto ErrorExit;
         }
      break;

      case PLI_AFX_START:
         /* start of the animation rendering */
         if (err=Start(in->v_param[0],
               in->f_param[0],in->f_param[1],
               in->f_param[2],in->l_param[0],
               in->l_param[1],in->l_param[2])) {
            goto ErrorExit;
         }
      break;

      case PLI_AFX_FINISH:
         /* end of the animation rendering */
         if (err=Finish(in->v_param[0],
               in->f_param[0],in->f_param[1],
               in->f_param[2],in->l_param[0],
               in->l_param[1],in->l_param[2])) {
            goto ErrorExit;
         }
      break;

      case PLI_AFX_PRE_RENDER:
         /* start of frame rendering */
         if (err=PreRender(in->v_param[0],
               in->f_param[0],in->f_param[1],
               in->f_param[2],in->l_param[0],
               in->l_param[1],in->l_param[2])) {
            goto ErrorExit;
         }
      break;

      case PLI_AFX_AFTER_RENDER:
         /* start of frame rendering */
         if (err=AfterRender(in->v_param[0],
               in->f_param[0],in->f_param[1],
               in->f_param[2],in->l_param[0],
               in->l_param[1],in->l_param[2])) {
            goto ErrorExit;
         }
      break;

      case PLI_AFX_GET_INFO:
         switch (in->l_param[0]) {
            case PLI_AFX_STORE_INFO:
               out->l_param[1]=1; /* we have one object to store */
            break;
            default:
               out->l_param[1]=0; /* unknown info messages */
            break;
         }
      break;

      case PLI_AFX_GET_OBJS:
         out->l_param[1]=1; /* 1 object */
         if (!RPI_afx_get_obj(HND,in->v_param[0],1,&out->v_param[0])) {
            err=PLUG_GET_VALUE_ERR; goto ErrorExit;
         }
      break;

      case PLI_AFX_SET_OBJS:
         if (!RPI_afx_set_int(HND,in->v_param[0],0,1)) { /* set resume flag */
            err=PLUG_GET_VALUE_ERR; goto ErrorExit;
         }
         if (!RPI_afx_set_obj(HND,in->v_param[0],1,in->v_param[1])) {
            err=PLUG_SET_VALUE_ERR; goto ErrorExit;
         }
      break;

      default:/* unknown message */
         return(RPI_MESSAGE_UNKNOWN);
      break;
   }

//ExitMessageHandler: /* display error message (if any) and return ok */
   /* jump to this label if your job is done and you want the plugin */
   /* to return the control to reflections. set the appropriate err code */
   /* to get an error message displayed, otherwise set err to PLUG_OK. */
   if (err) {
      GetErrorMessage(err,mess);
      strcpy(in->s_param[0],mess);
      out->l_param[0]=PLI_AFX_RET_ERR;
   }
   else {
      out->l_param[0]=PLI_AFX_RET_OK;
   }
   return(RPI_MESSAGE_PROCESSED);

ErrorExit: /* display error message (if any) and terminate */
   /* jump to this label if the plugin cannot intialize or a severe error */
   /* has occured and the plugin has to be unloaded. set the appropriate */
   /* err code to get an error message displayed. */
   GetErrorMessage(err,mess);
   strcpy(in->s_param[0],mess);
   out->l_param[0]=PLI_AFX_RET_ERR;
   return(RPI_MESSAGE_ERROR);
}

/*****************************************************************************/
