/*****************************************************************************/
/*                        monzoom plugin skeleton                            */
/*****************************************************************************/

/* title:    create plugin skeleton */
/* author:   Reinhard Grams  */
/* version:  1.0, 20.10.97 */
/* updated:  10.3.98 by RG */
/* version:  1.1, 8.12.99 */
/* updated:  14.1.2000 by RG */
/* updated:  3.3.2000 by RG */

/*****************************************************************************/
/* this is what you have to do to customize this skeleton: */
/* 1. change the name of the plugin and the config file as defined below */
/* 2. create a directory in "plugin\data" with the name of your plugin and */
/*    place the icons for your tool (if needed) in this directory */
/* 3. add vars to the PlugData config structure and the appropriate */
/*    entries in the DataDesc structure */
/* 4. fill at the specified places the RefPlugConnect() */
/*    and RefPlugExit() functions with your code */
/* 5. setup the structs and functions for the non modal requester */ 
/* 6. place the tool code in DoCreateTool() */ 
/* 7. test your new plugin and send a copy to Reflections support center */
/*****************************************************************************/
 
#include <stdio.h>
#include <stdlib.h>
#include <stddef.h>
#include <math.h>

/*****************************************************************************/
/* error codes and messages, add your own errors at the specified places */
 
#define PLUG_OK                  0

#define PLUG_INTERNAL_ERR        0
#define PLUG_NO_MEM_ERR          1
#define PLUG_OPEN_FILE_ERR       2
#define PLUG_READ_ERR            3
#define PLUG_WRITE_ERR           4
#define PLUG_FILE_ERR            5
#define PLUG_CONFIG_ERR          6
#define PLUG_CONFIG_OPEN_ERR     7
#define PLUG_OPEN_REQ_ERR        8
#define PLUG_GET_VALUE_ERR       9
#define PLUG_REF_DIR_ERR         10
#define PLUG_DLL_VERSION_ERR     11
#define PLUG_INSTALL_EVENT_ERR   12
#define PLUG_WORKING_ERR         13
/* add your error codes here */

#define ERROR_MESSAGES 14
/* increase this constant if you added error codes */

char  *mess_text[ERROR_MESSAGES]=
{
   "Interner Fehler",
   "Kein Speicher mehr frei",
   "Kann File nicht ffnen",
   "Kann Datei nicht lesen",
   "Kann Datei nicht schreiben",
   "Fehler beim Dateizugriff",
   "Fehlerhafte Konfigurationsdatei",
   "Kann Konfigurationsdatei nicht ffnen",
   "Kann Requester nicht ffnen",
   "Kann Werte nicht abfragen",
   "Kann Verzeichnis nicht ermitteln",
   "Bentige aktuelle MonPlugs.dll",
   "Kann Event nicht setzen",
   "Plugin ist bereits aktiv",
   /* add your error message here */
};

/*****************************************************************************/
/* global plugin handle needed for comunication with reflections */
/* valid between FPLUG_INIT_MESSAGE and FPLUG_CLOSE_MESSAGE */

void *HND=NULL;

/*****************************************************************************/
/* set this to 43 or less if you compile for an older version */

#define MONZOOM_VERSION    45
#define MONZOOM_REVISION   0

#define DLL_VERSION ((MONZOOM_VERSION<<16) | MONZOOM_REVISION)

/*****************************************************************************/
/* define this if you use the skeleton's standard config io */

#define PLUGIN_STANDARD_CONFIG_HANDLING

/*****************************************************************************/
/* undef this if you don't want to use the MonStubs.dll */

#define PLUGIN_STUBS_LIBRARY

/*****************************************************************************/
/* use 1 to get the stubs included, 0 otherwise */
/* this defs are used in "rgw_all.h" */

#define INCLUDE_GUI_STUBS           1
#define INCLUDE_PKL_STUBS           1
#define INCLUDE_GEO_EDIT_STUBS      0
#define INCLUDE_OPERATIONS_STUBS    0
#define INCLUDE_CREATE_STUBS        1
#define INCLUDE_OBJECT_STUBS        1
#define INCLUDE_GEO_STUBS           0
#define INCLUDE_EDGES_STUBS         0
#define INCLUDE_TRIANGLE_STUBS      0
#define INCLUDE_MATERIALS_STUBS     0
#define INCLUDE_MEMORY_STUBS        1
#define INCLUDE_FILE_STUBS          1
#define INCLUDE_POLYGON_STUBS       0
#define INCLUDE_PARTICLE_STUBS      0
#define INCLUDE_SPHERES_STUBS       0
#define INCLUDE_DATA_STUBS          0
#define INCLUDE_DISPLAY_STUBS       0
#define INCLUDE_AFTERBURNER_STUBS   0
#define INCLUDE_AFX_STUBS           0
#define INCLUDE_CAMERA_STUBS        0

/*****************************************************************************/
/* this includes all files needed */

#include <rgw_all.h>
#include <NMReq.c>

/*****************************************************************************/
/* this struct is initialized by this skeleton automaticly */
/* you can read this values after PLI_CONNECT at any time */

struct _info_ {

   char ref_path[260];        /* reflections working directory */
   char ref_temp[260];        /* reflections temp directory */
   long ref_version;          /* reflections version number */
   long ref_release;          /* reflections release number */
   long ref_revision;         /* reflections revision number */
   long ref_language;         /* the language of reflections */
   long scr_width;            /* width of the reflections display */
   long scr_height;           /* height of the reflections display */
   long scr_depth;            /* color depth of the reflections display */

} info = { "","",0,0,0,0,0,0,0 };

/*****************************************************************************/

#define PLUG_DATA_ENTRIES  5

typedef struct {

   char     string[32];
   float    floats[2];
   long     longs[2];
   short    shorts[2];
   long     cycle;
   /* replace with your own config vars */

} PlugData;

DataDesc descriptors[PLUG_DATA_ENTRIES] = {

   DATA_STRING,32,
   DATA_FLOAT,2,
   DATA_LONG,2,
   DATA_SHORT,2,
   DATA_LONG,1
   /* replace with your own config vars descriptors */
};

/*****************************************************************************/
/* this config struct is initialized by this skeleton automaticly */

#define PLUG_CONFIG_ID        "CFG" /* do not change this */
#define PLUG_CONFIG_VERSION   1     /* increase if you change config data format */

struct Config {

   char     id[4];    /* do not change this */
   long     version;  /* dito */
   long     menu_x;   /* dito */
   long     menu_y;   /* dito */
   PlugData data;     /* dito */

} user_config, default_config = {

   PLUG_CONFIG_ID,PLUG_CONFIG_VERSION,16,24,
   {
      "Willi",
      1.0F,2.0F,
      1,2,
      1,2,
      0,
      /* replace with your config defaults */
   },
};

struct Config *config=&default_config;
/* get your config values only from config-> */
/* it will be set to a loaded config if config file is valid */

/*****************************************************************************/
/* the name, version, filenames and help keyword of this plugin */

#define PLUG_NAME             "CreaSkel"
#define PLUG_VERSION          1
#define PLUG_REVISION         0

#ifdef WINDOWS
   #define CONFIG_FILENAME    "plugin\\data\\creaskel\\config"
#endif
#ifdef AMIGA
   #define CONFIG_FILENAME    "plugin/data/creaskel/config"
#endif

#define PLUG_HELP             "HELP_CREASKEL" /* help keyword */

#define PLUG_FILE_MODE        FILE_TYPE_INTEL /* byte order for config file */

/*****************************************************************************/
/* this messages are displayed after connection and before unloading */

#define PLUG_INSTALL_MESSAGE  "%s V%d.%d installiert unter V%ld.%ld.%ld"
#define PLUG_REMOVE_MESSAGE   "%s V%d.%d deaktiviert"

/*****************************************************************************/
/* default ids for your tools */

#define TOOL_ID1              1
#define TOOL_ID2              2

/*****************************************************************************/
/* the menu entries */

#define NR_OF_CREATE_ENTRIES 2

char *tool_names[NR_OF_CREATE_ENTRIES]= {

   "CreaSkel1",
   "CreaSkel2",
};

/*****************************************************************************/

#define REQ1_TITLE   "Skeleton"

/* change this defines to match your requester entries */
#define REQ1_STRINGS    1
#define REQ1_FLOATS     2 /* normal + extended */
#define REQ1_INTS       2 /* normal + extended */
#define REQ1_LINES      3
#define REQ1_TEXTS      1
#define REQ1_CYCLES     1
#define REQ1_EXTENDED   2 /* nr of extended ints and/or floats */

/* do not edit this */
#define REQ1_VARS       (REQ1_FLOATS+REQ1_INTS)
#define REQ1_ENTRIES    (REQ1_VARS+REQ1_STRINGS+REQ1_TEXTS+REQ1_CYCLES)
#define REQ1_ITEMS      (REQ1_ENTRIES+REQ1_LINES+REQ1_CYCLES)

/* maximum chars for requested strings */
#define REQ1_STRING_BUFFER_SIZE 64

/* place the name of the requester elements here */
char *req1_names[REQ1_ENTRIES] = {

   "String",
   "Int 1",
   "Int 2",
   "Floats",
   "Float 1",
   "Float 2",
   "Switch",
};

/* this is a cycle struct, one for each cycle */
Cycle req1_cycle=
{
  NULL,
  "Heinz","Hugo","Hubert",
};

/* the types of the requester elements, to be edited */
long req1_codes[REQ1_ITEMS] = {

   CODE_STRING,
   CODE_LINE,
   CODE_INT,
   CODE_INT_EXTENDED,
   CODE_TEXT,
   CODE_REAL,
   CODE_REAL_EXTENDED,
   CODE_LINE,
   CODE_CYCLE,3, /* three cycle elements */
   CODE_LINE,
};

/* a min and max for each integer or float extended element */
/* defined as floats */

float req1_minmax[REQ1_EXTENDED*2] = {

   0.0F,5.0F,
   0.0F,10.0F,
};

/* this needs no changes */
char req1_tmp_strings[REQ1_STRINGS][REQ1_STRING_BUFFER_SIZE];
long req1_tmp_ints[REQ1_INTS];
float req1_tmp_floats[REQ1_FLOATS];
long req1_tmp_cycles[REQ1_CYCLES];

/* temp storage for the requester values, to be edited */
void *req1_tmp_vars[REQ1_ENTRIES] = {
   
   &req1_tmp_strings[0],
   &req1_tmp_ints[0],
   &req1_tmp_ints[1],
   NULL, /* null for text elements */
   &req1_tmp_floats[0],
   &req1_tmp_floats[1],
   &req1_tmp_cycles[0],
};

/* this needs no changes */
char req1_org_strings[REQ1_STRINGS][REQ1_STRING_BUFFER_SIZE];
long req1_org_ints[REQ1_INTS];
float req1_org_floats[REQ1_FLOATS];
long req1_org_cycles[REQ1_CYCLES];

/* buffer for original values, to be edited */
void *req1_org_vars[REQ1_ENTRIES] = {
   
   &req1_org_strings[0],
   &req1_org_ints[0],
   &req1_org_ints[1],
   NULL, /* null for text elements */
   &req1_org_floats[0],
   &req1_org_floats[1],
   &req1_org_cycles[0],
};

/* this needs no changes */
void *req1_vars[REQ1_ENTRIES];
long req1_messages;

/*****************************************************************************/
/* globals to keep track of the status */

long tool_installed=0;
long tool_requester=0;
long ignore_delete=0;
RPI_OBJECT create_object=NULL;

/*****************************************************************************/
/* read the configuration file of the plugin */

   long
RefPlugReadConfig
   (void)
{
   long err=PLUG_OK;
   RPI_FILE file=NULL;
   char name[256];
   long item=0;
   char *data=(char *)&user_config.data;

   strcpy(name,info.ref_path); strcat(name,CONFIG_FILENAME);

   if (RPI_file_exist(HND,name)) {

      if (!(file=RPI_file_open_read_binary(HND,PLUG_FILE_MODE,name))) {
         err=PLUG_CONFIG_OPEN_ERR; goto ErrorExit;
      }
      if (!(RPI_file_read_binary(file,user_config.id,4))) {
         err=PLUG_READ_ERR; goto ErrorExit;
      }
      if (strcmp(user_config.id,PLUG_CONFIG_ID)) {
         err=PLUG_CONFIG_ERR; goto ErrorExit;
      }
      if (!(RPI_file_read_long(file,&user_config.version))) {
         err=PLUG_READ_ERR; goto ErrorExit;
      }
      if (user_config.version!=PLUG_CONFIG_VERSION) {
         err=PLUG_CONFIG_ERR; goto ErrorExit;
      }
      if (!(RPI_file_read_long(file,&user_config.menu_x))) {
         err=PLUG_READ_ERR; goto ErrorExit;
      }
      if (!(RPI_file_read_long(file,&user_config.menu_y))) {
         err=PLUG_READ_ERR; goto ErrorExit;
      }
      while (item<PLUG_DATA_ENTRIES) {

         if (!(data_read.functions[descriptors[item].type])(file,data,descriptors[item].count)) {
            err=PLUG_READ_ERR; goto ErrorExit;
         }
         data=&data[descriptors[item].count*data_size[descriptors[item].type]];
         item++;
      }
      config=&user_config; /* all values readed, config is valid */
   }
   
ErrorExit:
   if (file) RPI_file_close(file);
   return(err);
}

/*****************************************************************************/
/* write the configuration file of the plugin */

   long
RefPlugWriteConfig
   (void)
{
   long err=PLUG_OK;
   RPI_FILE file=NULL;
   char name[256];
   long item=0;
   char *data=(char *)&config->data;

   strcpy(name,info.ref_path); strcat(name,CONFIG_FILENAME);

   if (!(file=RPI_file_open_write_binary(HND,PLUG_FILE_MODE,name))) {
      err=PLUG_CONFIG_OPEN_ERR; goto ErrorExit;
   }
   if (!(RPI_file_write_binary(file,config->id,4))) {
      err=PLUG_WRITE_ERR; goto ErrorExit;
   }
   if (!(RPI_file_write_long(file,config->version))) {
      err=PLUG_WRITE_ERR; goto ErrorExit;
   }
   if (!(RPI_file_write_long(file,config->menu_x))) {
      err=PLUG_WRITE_ERR; goto ErrorExit;
   }
   if (!(RPI_file_write_long(file,config->menu_y))) {
      err=PLUG_WRITE_ERR; goto ErrorExit;
   }
   while (item<PLUG_DATA_ENTRIES) {

      if (!(data_write.functions[descriptors[item].type])(file,data,descriptors[item].count)) {
         err=PLUG_WRITE_ERR; goto ErrorExit;
      }
      data=&data[descriptors[item].count*data_size[descriptors[item].type]];
      item++;
   }

ErrorExit:
   if (file) RPI_file_close(file);
   return(err);
}

/*****************************************************************************/
/* get the error message of error code */

   void
GetErrorMessage
   (long err,char *buffer)
{
   if (err>0 && err<ERROR_MESSAGES) 
      sprintf(buffer,"%s: %s",PLUG_NAME,mess_text[err]);
   else 
      sprintf(buffer,"%s: %s %ld",PLUG_NAME,mess_text[PLUG_INTERNAL_ERR],err);
}

/*****************************************************************************/
/* this function is called at first when PLI_CONNECT arrives */

   long 
RefPlugConnectInit
   (void)
{
   long err=PLUG_OK;
   char mess[128];

   #ifdef PLUGIN_STUBS_LIBRARY   
   unsigned long dll_version;
   dll_version=MonPlugsGetVersion();
   if (dll_version<DLL_VERSION) {
      err=PLUG_DLL_VERSION_ERR; goto ErrorExit;
   }
   #endif

   /* request pathes, screen, version and language values */
   if (!RPI_get_ref_char_info(HND,REF_DIR,info.ref_path)) {
      err=PLUG_REF_DIR_ERR; goto ErrorExit;
   }
   if (!RPI_get_ref_char_info(HND,REF_TEMP_DIR,info.ref_temp)) {
      err=PLUG_REF_DIR_ERR; goto ErrorExit;
   }
   if (!RPI_get_ref_info(HND,DISP_DX,&info.scr_width)) {
      err=PLUG_GET_VALUE_ERR; goto ErrorExit;
   }
   if (!RPI_get_ref_info(HND,DISP_DY,&info.scr_height)) {
      err=PLUG_GET_VALUE_ERR; goto ErrorExit;
   }
   if (!RPI_get_ref_info(HND,DISP_DEPTH,&info.scr_depth)) {
      err=PLUG_GET_VALUE_ERR; goto ErrorExit;
   }
   if (!RPI_get_ref_info(HND,VERSION,&info.ref_version)) {
      err=PLUG_GET_VALUE_ERR; goto ErrorExit;
   }
   if (!RPI_get_ref_info(HND,RELEASE,&info.ref_release)) {
      err=PLUG_GET_VALUE_ERR; goto ErrorExit;
   }
   if (!RPI_get_ref_info(HND,REVISION,&info.ref_revision)) {
      err=PLUG_GET_VALUE_ERR; goto ErrorExit;
   }
   if (!RPI_get_ref_info(HND,LANGUAGE,&info.ref_language)) {
      err=PLUG_GET_VALUE_ERR; goto ErrorExit;
   }
   if (err=RefPlugReadConfig()) {
      goto ErrorExit;
   }
   sprintf(mess,PLUG_INSTALL_MESSAGE,PLUG_NAME,PLUG_VERSION,PLUG_REVISION,
      info.ref_version,info.ref_release,info.ref_revision);
   RPI_print(HND,mess);

ErrorExit:
   return(err);
}

/*****************************************************************************/
/* this function is called at last when PLI_CONNECT arrives */

   long 
RefPlugConnect
   (void)
{
   long err=PLUG_OK;

   /* place your setup code here */

   return(err);
}

/*****************************************************************************/
/* this function is called when PLI_CLOSE_RESOURCES arrives */

   long
RefPlugCloseResources
   (void)
{
   long err=PLUG_OK;
   char mess[128];

   /* place your cleanup code here */

   sprintf(mess,PLUG_REMOVE_MESSAGE,PLUG_NAME,PLUG_VERSION,PLUG_REVISION);
   RPI_print(HND,mess);
   err=RefPlugWriteConfig();

   return(err);
}

/*****************************************************************************/
/* used to delete our own object (the create_object) */

   void
RefPlugRemoveToolObject
   (void)
{
   long lock;

   if (create_object) {
      ignore_delete=1;
      RPI_obj_unlock(HND,create_object,&lock);
      RPI_obj_free(HND,create_object);
      create_object=NULL;
      ignore_delete=0;
   }
}

/*****************************************************************************/
/* the user has edited an element of the requester */

   long
HandleNMRequester
   (long tool_id)
{
   long err=PLUG_OK;
   long changed;

   changed=GetNMRequesterAlteredVariable(REQ1_ITEMS,req1_codes,req1_vars,req1_tmp_vars);

   /* check if requester is still under construction */
   if (!req1_messages) goto ErrorExit;
 
   /* if a value has been edited */
   if (changed>=0) {

      switch (changed) {
         case 0:
            RPI_print(HND,"String edited");
         break;
         case 1:
            RPI_print(HND,"Int 1 edited");
         break;
         case 2:
            RPI_print(HND,"Int 2 edited");
         break;
         case 4:
            RPI_print(HND,"Float 1 edited");
         break;
         case 5:
            RPI_print(HND,"Float 2 edited");
         break;
         case 6:
            RPI_print(HND,"Cycle edited");
         break;
      }
   }

ErrorExit:
   return(err);
}

/*****************************************************************************/

   long
HandleCancelNMRequester
   (long tool_id)
{
   long err=PLUG_OK;

   /* user has canceled requester, restore original values */
   CancelNMRequester(REQ1_ITEMS,req1_codes,req1_vars,req1_org_vars);
   /* if object has been created, remove it */
   if (create_object) {
      RefPlugRemoveToolObject();
      RPI_plot_pkl(HND);
   }
   tool_requester=0;
   return(err);
}

/*****************************************************************************/

   long
HandleExitNMRequester
   (long tool_id)
{
   long err=PLUG_OK;
   long lock;

   /* user has closed requester, perform cleanup if neccessary */
   if (create_object) {
      RPI_obj_unlock(HND,create_object,&lock);
      create_object=NULL;
   }
   tool_requester=0;
   return(err);
}

/*****************************************************************************/
/* this is the beef! this function gets called when the user selected */
/* one of our created menu entries */

   long
DoCreateTool
   (long tool_id)
{
   long err=PLUG_OK;
   PlugData *data=&config->data;

   /* set the pointers to the requester variables */
   req1_vars[0]=data->string;     /* first element is a string */
   req1_vars[1]=&data->longs[0];  /* first integer */
   req1_vars[2]=&data->longs[1];  /* second integer */
   req1_vars[3]=NULL; /* null for text elements */
   req1_vars[4]=&data->floats[0]; /* first float */
   req1_vars[5]=&data->floats[1]; /* second float */
   req1_vars[6]=&req1_cycle; /* cycle structure */
   req1_cycle.select=&data->cycle; /* ptr in structure */

   create_object=NULL; /* just to avoid panic */
   tool_requester=0;

   if (err=CreateNMRequester(REQ1_TITLE,TOOL_ID1,REQ1_ITEMS,
         &req1_messages,req1_codes,req1_vars,req1_tmp_vars,req1_org_vars,
         req1_names,req1_minmax,INTERACTIVE_REQUESTER)) {
      goto ErrorExit;
   }
   tool_requester=1;

ErrorExit:
   return(err);
}

/*****************************************************************************/
/* this is the one and only entry of our plugin for reflections */
/* you may add case switches to process other messages */

   long REFPROG
RPI_message
   (PTR0 PLI_param *in,PTR1 PLI_param *out)
{
   long err=PLUG_OK;
   char mess[128];
   long lock,tool;

   /* obsolete in further versions */
   out->l_param[0]=TOOL_OK;

   #ifdef AMIGA
   if (!libs_installed) return(RPI_MESSAGE_UNKNOWN);
   #endif

   switch(in->id) {

      case FPLUG_INIT_MESSAGE: 
         /* only initialize our plugin, we get connected later */         
         HND=in->v_param[0]; /* get global plug handle for further use */
      break;

      case PLI_CONNECT:
          /* open amiga libraries */
         #include <AmiOpen.c>
         /* call plugin init function */
         if (err=RefPlugConnectInit()) {
            goto ErrorExit;
         }
         /* the basic things are set up, now tell reflections */
         /* what kind of messages we are waiting for */
         for (tool=NR_OF_CREATE_ENTRIES-1;tool>=0;tool--) {
            if (!RPI_add_create_tool(HND,tool_names[tool],TOOL_ID1+tool)) {
               err=PLUG_INSTALL_EVENT_ERR; goto ErrorExit;
            }
         }
         tool_installed=1;
         /* call plugin connect function */
         if (err=RefPlugConnect()) {
            goto ErrorExit;
         }
      break;

      case PLI_TOUCH:
         /* user has selected the plugin menu entry a second time */
         RPI_free_myself(HND);
      break;

      case PLI_CLOSE_RESOURCES:
         /* reflections is going to remove us, call shutdown function */
         if (tool_installed) {
            for (tool=NR_OF_CREATE_ENTRIES-1;tool>=0;tool--) {
               RPI_sub_create_tool(HND,TOOL_ID1+tool);
            }
            tool_installed=0;
         }
         if (create_object) {
            RPI_obj_unlock(HND,create_object,&lock);
            create_object=NULL;
         }
 			if (tool_requester) {
            RPI_close_nonmodal_requester(HND,TOOL_ID1);
      		tool_requester=0;
	      }
         /* call plugin exit function */
         err=RefPlugCloseResources();
      break;

      case FPLUG_CLOSE_MESSAGE:
         /* we are going to be removed from memory, the shop is closed! */
         HND=NULL;
         /* close the amiga libraries */
         #include <AmiClose.c>
      break;

      case PLI_DO_CREATE_TOOL:
         /* now its time to create whatever has to be created */
         if (create_object || tool_requester) {
            err=PLUG_WORKING_ERR; goto ExitMessageHandler;
         }
         if (!RPI_obj_inform(HND,INFORM_OBJ_DEL)) {
            err=PLUG_INSTALL_EVENT_ERR; goto ErrorExit;
         }
         if (err=DoCreateTool(in->l_param[0])) {
            goto ExitMessageHandler;
         }
      break;

      case PLI_NM_REQUESTER_CHANGED:
         /* user has edited a requester entry */
         if (err=HandleNMRequester(in->l_param[0])) {
            goto ErrorExit;
         }
      break;

      case PLI_NM_REQUESTER_CANCEL:
         /* user has canceled requester */
         if (err=HandleCancelNMRequester(in->l_param[0])) {
            goto ErrorExit;
         }
      break;

      case PLI_NM_REQUESTER_OK:
         /* user has closed the requester */
         if (err=HandleExitNMRequester(in->l_param[0])) {
            goto ErrorExit;
         }
      break;

      case PLI_OBJ_EVENT:
         /* check if our object has been deleted */
         if (!ignore_delete && in->l_param[0]==INFORM_OBJ_DEL && in->v_param[0]==create_object) {
   			if (tool_requester) {
               RPI_close_nonmodal_requester(HND,TOOL_ID1);
		   		tool_requester=0;
			   }
            create_object=NULL;
         }
      break;

      default:/* other messages */
         return(RPI_MESSAGE_UNKNOWN);
      break;
   }

ExitMessageHandler: /* display error message (if any) and return ok */
   /* jump to this label if your job is done and you want the plugin */
   /* to return the control to reflections. set the appropriate err code */
   /* to get an error message displayed, otherwise set err to PLUG_OK. */
   if (err) {
      GetErrorMessage(err,mess);
      RPI_message_win(HND,mess);
   }
   return(RPI_MESSAGE_PROCESSED);

ErrorExit: /* display error message and die */
   /* jump to this label if the plugin cannot intialize or a severe error */
   /* has occured and the plugin has to be unloaded. set the appropriate */
   /* err code to get an error message displayed, PLUG_OK is not allowed. */
   GetErrorMessage(err,mess);
   RPI_message_win(HND,mess);   
   RPI_free_myself(HND);
   return(RPI_MESSAGE_ERROR);
}

/*****************************************************************************/
