/*****************************************************************************/
/*                        monzoom plugin skeleton                            */
/*****************************************************************************/

/* title:    geo tool plugin skeleton */
/* author:   Reinhard Grams  */
/* version:  1.0, 18.9.97 */
/* updated:  10.3.98 by RG */
/* updated:  14.1.2000 by RG */
/* updated:  3.3.2000 by RG */

/*****************************************************************************/
/* this is what you have to do to customize this skeleton: */
/* 1. change the name of the plugin, the filenames for the tool icons and */
/*    the config file as defined below */
/* 2. create a directory in "plugin\data" with the name of your plugin and */
/*    place the icons for your tool (if needed) in this directory */
/* 3. add vars to the PlugData config structure and the appropriate */
/*    entries in the DataDesc structure */
/* 4. fill at the specified places the RefPlugConnect() */
/*    and RefPlugExit() functions with your code */
/* 5. place your code in RPI_message() at the specified locations */
/* 6. test your new plugin and send a copy to Reflections support center */
/*****************************************************************************/
 
#include <stdio.h>
#include <stdlib.h>
#include <stddef.h>
#include <math.h>

/*****************************************************************************/
/* error codes and messages, add your own errors at the specified places */
 
#define PLUG_OK                  0

#define PLUG_INTERNAL_ERR        0
#define PLUG_NO_MEM_ERR          1
#define PLUG_OPEN_FILE_ERR       2
#define PLUG_READ_ERR            3
#define PLUG_WRITE_ERR           4
#define PLUG_FILE_ERR            5
#define PLUG_CONFIG_ERR          6
#define PLUG_CONFIG_OPEN_ERR     7
#define PLUG_OPEN_REQ_ERR        8
#define PLUG_GET_VALUE_ERR       9
#define PLUG_REF_DIR_ERR         10
#define PLUG_DLL_VERSION_ERR     11
#define PLUG_INSTALL_EVENT_ERR   12
#define PLUG_INSTALL_ICON_ERR    13
#define PLUG_NO_OBJECT_ERR       14
/* add your error codes here */

#define ERROR_MESSAGES 15
/* increase this constant if you added error codes */

char  *mess_text[ERROR_MESSAGES]=
{
   "Interner Fehler",
   "Kein Speicher mehr frei",
   "Kann File nicht ffnen",
   "Kann Datei nicht lesen",
   "Kann Datei nicht schreiben",
   "Fehler beim Dateizugriff",
   "Fehlerhafte Konfigurationsdatei",
   "Kann Konfigurationsdatei nicht ffnen",
   "Kann Requester nicht ffnen",
   "Kann Werte nicht abfragen",
   "Kann Verzeichnis nicht ermitteln",
   "Bentige aktuelle MonPlugs.dll",
   "Kann Event nicht setzen",
   "Kann Icon nicht erzeugen ",
   "Kein Objekt angewhlt ",
/* add your error message here */
};

/*****************************************************************************/
/* global plugin handle needed for comunication with reflections */
/* valid between FPLUG_INIT_MESSAGE and FPLUG_CLOSE_MESSAGE */

void *HND=NULL;

/*****************************************************************************/
/* set this to 43 or less if you compile for an older version */

#define MONZOOM_VERSION    45
#define MONZOOM_REVISION   0

#define DLL_VERSION ((MONZOOM_VERSION<<16) | MONZOOM_REVISION)

/*****************************************************************************/
/* define this if you use the skeleton's standard config io */

#define PLUGIN_STANDARD_CONFIG_HANDLING

/*****************************************************************************/
/* undef this if you don't want to use the MonStubs.dll */

#define PLUGIN_STUBS_LIBRARY

/*****************************************************************************/
/* use 1 to get the stubs included, 0 otherwise */
/* this defs are used in "rgw_all.h" */

#define INCLUDE_GUI_STUBS           1
#define INCLUDE_PKL_STUBS           0
#define INCLUDE_GEO_EDIT_STUBS      1
#define INCLUDE_OPERATIONS_STUBS    0
#define INCLUDE_CREATE_STUBS        0
#define INCLUDE_OBJECT_STUBS        0
#define INCLUDE_GEO_STUBS           0
#define INCLUDE_EDGES_STUBS         0
#define INCLUDE_TRIANGLE_STUBS      0
#define INCLUDE_MATERIALS_STUBS     0
#define INCLUDE_MEMORY_STUBS        1
#define INCLUDE_FILE_STUBS          1
#define INCLUDE_POLYGON_STUBS       0
#define INCLUDE_PARTICLE_STUBS      0
#define INCLUDE_SPHERES_STUBS       0
#define INCLUDE_DATA_STUBS          0
#define INCLUDE_DISPLAY_STUBS       0
#define INCLUDE_AFTERBURNER_STUBS   0
#define INCLUDE_AFX_STUBS           0
#define INCLUDE_CAMERA_STUBS        0

#include <rgw_all.h>

/*****************************************************************************/
/* this struct is initialized by this skeleton automaticly */
/* you can read this values after FPLUG_INIT_MESSAGE at any time */

struct _info_ {

   char ref_path[260];        /* reflections working directory */
   char ref_temp[260];        /* reflections temp directory */
   long ref_version;          /* reflections version number */
   long ref_release;          /* reflections release number */
   long ref_revision;         /* reflections revision number */
   long ref_language;         /* the language of reflections */
   long scr_width;            /* width of the reflections display */
   long scr_height;           /* height of the reflections display */
   long scr_depth;            /* color depth of the reflections display */

} info = { "","",0,0,0,0,0,0,0 };

/*****************************************************************************/
/* edit this two structures to get your config data loaded & stored */

#define PLUG_DATA_ENTRIES  1

typedef struct {

   float    dummy;

} PlugData;

/* data type, count */
DataDesc descriptors[PLUG_DATA_ENTRIES] = {

   DATA_FLOAT,1,
};

/*****************************************************************************/
/* this config struct is initialized by this skeleton automaticly */

#define PLUG_CONFIG_ID "CFG"        /* do not change this */
#define PLUG_CONFIG_VERSION   1     /* increase if you change config data format */

struct _config_ {

   char     id[4];   /* do not edit this */
   long     version; /* dito */
   long     menu_x;  /* dito */
   long     menu_y;  /* dito */
   PlugData data;    /* dito */

} user_config, default_config = { 
   
   PLUG_CONFIG_ID,PLUG_CONFIG_VERSION,16,24, 
   { 0.0F, }    
};/* replace with your config defaults */

struct _config_ *config=&default_config;
/* get your config values only from config-> */
/* it will be set to a loaded config if config file is valid */

/*****************************************************************************/
/* the name, version, filenames and help keyword of this plugin */

#define PLUG_NAME             "ToolSkel"
#define PLUG_VERSION          1
#define PLUG_REVISION         0

#ifdef WINDOWS
   #define ICON1_FILENAME     "plugin\\data\\toolskel\\icon1.iff"
   #define SELECT1_FILENAME   "plugin\\data\\toolskel\\select1.iff"
   #define CONFIG_FILENAME    "plugin\\data\\toolskel\\config"
#endif
#ifdef AMIGA
   #define ICON1_FILENAME     "plugin/data/toolskel/icon1.iff"
   #define SELECT1_FILENAME   "plugin/data/toolskel/select1.iff"
   #define CONFIG_FILENAME    "plugin/data/toolskel/config"
#endif

#define PLUG_HELP             "HELP_TOOLSKEL" /* help keyword */

#define PLUG_FILE_MODE        FILE_TYPE_INTEL /* byte order for config file */

#define PLUG_OBJECT_ID        DREIECKS_OBJEKT_ID /* known object type */

/*****************************************************************************/
/* this messages are displayed after connection and before unloading */

#define PLUG_INSTALL_MESSAGE  "%s V%d.%d installiert unter V%ld.%ld.%ld"
#define PLUG_REMOVE_MESSAGE   "%s V%d.%d deaktiviert"

/*****************************************************************************/
/* default ids for your tools */

#define TOOL_ID1              1
#define TOOL_ID2              2

/*****************************************************************************/
/* globals to keep track of the status */

long icon_installed=0;
long geo_running=0;
RPI_OBJECT interactive_object=NULL;

/*****************************************************************************/
/* read the configuration file of the plugin */

   long
RefPlugReadConfig
   (void)
{
   long err=PLUG_OK;
   RPI_FILE file=NULL;
   char name[256];
   long item=0;
   char *data=(char *)&user_config.data;

   strcpy(name,info.ref_path); strcat(name,CONFIG_FILENAME);

   if (RPI_file_exist(HND,name)) {

      if (!(file=RPI_file_open_read_binary(HND,PLUG_FILE_MODE,name))) {
         err=PLUG_CONFIG_OPEN_ERR; goto ErrorExit;
      }
      if (!(RPI_file_read_binary(file,user_config.id,4))) {
         err=PLUG_READ_ERR; goto ErrorExit;
      }
      if (strcmp(user_config.id,PLUG_CONFIG_ID)) {
         err=PLUG_CONFIG_ERR; goto ErrorExit;
      }
      if (!(RPI_file_read_long(file,&user_config.version))) {
         err=PLUG_READ_ERR; goto ErrorExit;
      }
      if (user_config.version!=PLUG_CONFIG_VERSION) {
         err=PLUG_CONFIG_ERR; goto ErrorExit;
      }
      if (!(RPI_file_read_long(file,&user_config.menu_x))) {
         err=PLUG_READ_ERR; goto ErrorExit;
      }
      if (!(RPI_file_read_long(file,&user_config.menu_y))) {
         err=PLUG_READ_ERR; goto ErrorExit;
      }
      while (item<PLUG_DATA_ENTRIES) {

         if (!(data_read.functions[descriptors[item].type])(file,data,descriptors[item].count)) {
            err=PLUG_READ_ERR; goto ErrorExit;
         }
         data=&data[descriptors[item].count*data_size[descriptors[item].type]];
         item++;
      }
      config=&user_config; /* all values readed, config is valid */
   }
   
ErrorExit:
   if (file) RPI_file_close(file);
   return(err);
}

/*****************************************************************************/
/* write the configuration file of the plugin */

   long
RefPlugWriteConfig
   (void)
{
   long err=PLUG_OK;
   RPI_FILE file=NULL;
   char name[256];
   long item=0;
   char *data=(char *)&config->data;

   strcpy(name,info.ref_path); strcat(name,CONFIG_FILENAME);

   if (!(file=RPI_file_open_write_binary(HND,PLUG_FILE_MODE,name))) {
      err=PLUG_CONFIG_OPEN_ERR; goto ErrorExit;
   }
   if (!(RPI_file_write_binary(file,config->id,4))) {
      err=PLUG_WRITE_ERR; goto ErrorExit;
   }
   if (!(RPI_file_write_long(file,config->version))) {
      err=PLUG_WRITE_ERR; goto ErrorExit;
   }
   if (!(RPI_file_write_long(file,config->menu_x))) {
      err=PLUG_WRITE_ERR; goto ErrorExit;
   }
   if (!(RPI_file_write_long(file,config->menu_y))) {
      err=PLUG_WRITE_ERR; goto ErrorExit;
   }
   while (item<PLUG_DATA_ENTRIES) {

      if (!(data_write.functions[descriptors[item].type])(file,data,descriptors[item].count)) {
         err=PLUG_WRITE_ERR; goto ErrorExit;
      }
      data=&data[descriptors[item].count*data_size[descriptors[item].type]];
      item++;
   }

ErrorExit:
   if (file) RPI_file_close(file);
   return(err);
}

/*****************************************************************************/
/* get the error message of error code */

   void
GetErrorMessage
   (long err,char *buffer)
{
   if (err>0 && err<ERROR_MESSAGES) 
      sprintf(buffer,"%s: %s",PLUG_NAME,mess_text[err]);
   else 
      sprintf(buffer,"%s: %s %ld",PLUG_NAME,mess_text[PLUG_INTERNAL_ERR],err);
}

/*****************************************************************************/
/* this function is called at first when PLI_CONNECT arrives */

   long 
RefPlugConnectInit
   (void)
{
   long err=PLUG_OK;
   char mess[128];

   #ifdef PLUGIN_STUBS_LIBRARY   
   unsigned long dll_version;
   dll_version=MonPlugsGetVersion();
   if (dll_version<DLL_VERSION) {
      err=PLUG_DLL_VERSION_ERR; goto ErrorExit;
   }
   #endif

  /* request pathes, screen, version and language values */
   if (!RPI_get_ref_char_info(HND,REF_DIR,info.ref_path)) {
      err=PLUG_REF_DIR_ERR; goto ErrorExit;
   }
   if (!RPI_get_ref_char_info(HND,REF_TEMP_DIR,info.ref_temp)) {
      err=PLUG_REF_DIR_ERR; goto ErrorExit;
   }
   if (!RPI_get_ref_info(HND,DISP_DX,&info.scr_width)) {
      err=PLUG_GET_VALUE_ERR; goto ErrorExit;
   }
   if (!RPI_get_ref_info(HND,DISP_DY,&info.scr_height)) {
      err=PLUG_GET_VALUE_ERR; goto ErrorExit;
   }
   if (!RPI_get_ref_info(HND,DISP_DEPTH,&info.scr_depth)) {
      err=PLUG_GET_VALUE_ERR; goto ErrorExit;
   }
   if (!RPI_get_ref_info(HND,VERSION,&info.ref_version)) {
      err=PLUG_GET_VALUE_ERR; goto ErrorExit;
   }
   if (!RPI_get_ref_info(HND,RELEASE,&info.ref_release)) {
      err=PLUG_GET_VALUE_ERR; goto ErrorExit;
   }
   if (!RPI_get_ref_info(HND,REVISION,&info.ref_revision)) {
      err=PLUG_GET_VALUE_ERR; goto ErrorExit;
   }
   if (!RPI_get_ref_info(HND,LANGUAGE,&info.ref_language)) {
      err=PLUG_GET_VALUE_ERR; goto ErrorExit;
   }
   if (err=RefPlugReadConfig()) {
      goto ErrorExit;
   }
   sprintf(mess,PLUG_INSTALL_MESSAGE,PLUG_NAME,PLUG_VERSION,PLUG_REVISION,
      info.ref_version,info.ref_release,info.ref_revision);
   RPI_print(HND,mess);

ErrorExit:
   return(err);
}

/*****************************************************************************/
/* this function is called at last when PLI_CONNECT arrives */

   long 
RefPlugConnect
   (void)
{
   long err=PLUG_OK;

   /* place your setup code here */

   return(err);
}

/*****************************************************************************/
/* this function is called when PLI_CLOSE_RESOURCES arrives */

   long
RefPlugCloseResources
   (void)
{
   long err=PLUG_OK;
   char mess[128];

   /* place your cleanup code here */

   sprintf(mess,PLUG_REMOVE_MESSAGE,PLUG_NAME,PLUG_VERSION,PLUG_REVISION);
   RPI_print(HND,mess);
   err=RefPlugWriteConfig();

   return(err);
}

/*****************************************************************************/
/* this is the one and only entry of our plugin for reflections */
/* you may add case switches to process other messages */

   long REFPROG
RPI_message
   (PTR0 PLI_param *in,PTR1 PLI_param *out)
{
   long err=PLUG_OK;
   char mess[128],pic[256],sel[256];
   long event,tool_id,display_id,mouse_x,mouse_y;
   RPI_OBJECT object;

   #ifdef AMIGA
   if (!libs_installed) return(RPI_MESSAGE_UNKNOWN);
   #endif

   switch(in->id) {

      case FPLUG_INIT_MESSAGE: 
         /* only initialize our plugin, we get connected later */         
         HND=in->v_param[0]; /* get global plug handle for further use */
      break;

      case PLI_CONNECT:
         /* open amiga libraries */
         #include <AmiOpen.c>
         /* call plugin init function */
         if (err=RefPlugConnectInit()) {
            goto ErrorExit;
         }
         /* the basic things are set up, now tell reflections */
         /* what kind of messages we are waiting for */
         if (!RPI_geo_edit_inform(HND,INFORM_START_GEO_EDIT|INFORM_CLOSE_GEO_EDIT)) {
            err=PLUG_INSTALL_EVENT_ERR; goto ErrorExit;
         }
         /* add icon in x-tra window */
         strcpy(pic,info.ref_path); strcat(pic,ICON1_FILENAME);
         strcpy(sel,info.ref_path); strcat(sel,SELECT1_FILENAME);
         if (!RPI_geo_edit_add_interactive_tool(HND,pic,sel,
               TOOL_ID1,PLUG_OBJECT_ID)) {
            err=PLUG_INSTALL_ICON_ERR; goto ErrorExit;
         }
         icon_installed=1;
         /* call plugin connect function */
         if (err=RefPlugConnect()) {
            goto ErrorExit;
         }
      break;

      case PLI_TOUCH:
         /* user has selected the plugin menu entry a second time */
         RPI_free_myself(HND);
      break;

      case PLI_CLOSE_RESOURCES:
         /* reflections is going to remove us, do cleanup */
         if (icon_installed) {
            RPI_geo_edit_sub_tool(HND,TOOL_ID1);
            icon_installed=0;
         }
         /* call plugin cleanup function */
         err=RefPlugCloseResources();
      break;

      case FPLUG_CLOSE_MESSAGE:
         /* we are going to be removed from memory, the shop is closed! */
         HND=NULL;
         /* close the amiga libraries */
         #include <AmiClose.c>
      break;

		case PLI_GEO_EDIT_TOOL_INFO: /* user requests help */
			strcpy(in->s_param[0],PLUG_HELP); /* set help keyword */
			out->l_param[0]=2; /* we wish to display the help file */
		break;

      case PLI_GEO_EDIT_EVENT:
         /* geo editor opened or closed */
         event=in->l_param[0];  
         object=in->v_param[0]; /* can be NULL!! */

         switch(event) {
            case INFORM_START_GEO_EDIT: /* the geo-editor has been started */
               if (!geo_running) {
                  geo_running=1;
                  /* add code here and remove next line */
                  RPI_message_win(HND,"geo editor started");
               }
            break;
            case INFORM_CLOSE_GEO_EDIT: /* the geo-editor will be closed */
               if (geo_running) {
                  geo_running=0;
                  /* add code here and remove next line */
                  RPI_message_win(HND,"geo editor closed");
               }
            break;
            default: /* other messages */
               return(RPI_MESSAGE_UNKNOWN);
            break;
         }
      break;

      case PLI_GEO_EDIT_INTERACTIVE_TOOL_PRESSED:
         /* the user pressed the tool icon */
         interactive_object=NULL;
         if (!RPI_geo_edit_get_selected_obj(HND,&interactive_object)) {
            err=PLUG_NO_OBJECT_ERR; goto ExitMessageHandler;
         }
         /* place your code here and remove the next line */
         RPI_message_win(HND,"interactive tool started");
      break;

      case PLI_GEO_EDIT_INTERACTIVE_TOOL_RELEASED:
         /* the tool icon has been released */
         interactive_object=NULL;

         /* place your code here and remove the next line */
         RPI_message_win(HND,"interactive tool released");
      break;

      case PLI_GEO_EDIT_INTERACTIVE_TOOL_LMOUSE_DOWN:
         /* the user pressed the mouse button */
         if (interactive_object) { /* check if there is a valid object */
            tool_id=in->l_param[0];
            display_id=in->l_param[1];
            mouse_x=in->l_param[2];
            mouse_y=in->l_param[3];
            /* place your mouse down handler here */
         }
      break;

      case PLI_GEO_EDIT_INTERACTIVE_TOOL_LMOUSE_MOVE:
         /* the user moved the mouse while pressing the mouse button */
         if (interactive_object) {  /* check if there is a valid object */
            tool_id=in->l_param[0];
            display_id=in->l_param[1];
            mouse_x=in->l_param[2];
            mouse_y=in->l_param[3];
            /* place your mouse move handler here */
         }
      break;

      case PLI_GEO_EDIT_INTERACTIVE_TOOL_LMOUSE_UP:
         /* the user released the mouse button */
         if (interactive_object) {  /* check if there is a valid object */
            tool_id=in->l_param[0];
            display_id=in->l_param[1];
            mouse_x=in->l_param[2];
            mouse_y=in->l_param[3];
            /* place your mouse up handler here */
         }
      break;

      default:/* other messages */
         return(RPI_MESSAGE_UNKNOWN);
      break;
   }

ExitMessageHandler: /* display error message (if any) and return ok */
   /* jump to this label if your job is done and you want the plugin */
   /* to return the control to reflections. set the appropriate err code */
   /* to get an error message displayed, otherwise set err to PLUG_OK. */
   if (err) {
      GetErrorMessage(err,mess);
      RPI_message_win(HND,mess);
   }   
   return(RPI_MESSAGE_PROCESSED);

ErrorExit: /* display error message and die */
   /* jump to this label if the plugin cannot intialize or a severe error */
   /* has occured and the plugin has to be unloaded. set the appropriate */
   /* err code to get an error message displayed, PLUG_OK is not allowed. */
   GetErrorMessage(err,mess);
   RPI_message_win(HND,mess);   
   RPI_free_myself(HND);
   return(RPI_MESSAGE_ERROR);
}

/*****************************************************************************/
